import os
import string,re
from Warning import Warning

class VisualStudio:

	def __init__(self, project=None, target=None, directory=None, **extras):
		"""Project is the filename, like MD.dsp.
		Target is something like "MD - Win32 Debug"
		"""
		self.project = project
		self.target = target
		self.directory = directory
		self._OutputFileFromTarget( target )

	def Execute(self, rebuild=None,debug=None):
		"""Directory is where the makefile is.  The target specifier has
		two parts.  The first part is the workspace filename.  The second part
		is the name of the Visual Studio target."""

		curDir = os.getcwd()
		os.chdir( self.directory ) # /REBUILD
		if debug: print "VisualStudio:Execute:Removing %s" % self.outputFile
		try:
			os.remove(self.outputFile)
		except:
			if debug: print "VisualStudio:Execute:Failed to remove %s" % self.outputFile
			pass
		
		if rebuild:
			line = 'msdev %s /MAKE "%s" /OUT %s /REBUILD' % (self.project,
					self.target, self.outputFile)
		else:
			line = 'msdev %s /MAKE "%s" /OUT %s' % (self.project,
					self.target, self.outputFile)
		print line,"...",
		try:
			os.system( line )
			size = os.stat( self.outputFile )[6]
		except:
			size = 0
		print "Done"
		os.chdir( curDir )
		if not size:
			if debug: print 'Could not execute Visual Studio'
			raise IOError('Could not execute Visual Studio.')

	def Clean(self):
		curDir = os.getcwd()
		os.chdir( directory )
		# This won't run as is.  Get Execute() working first.
		os.system('msdev %s.dsp /MAKE "%s - Win32 %s"  /CLEAN')
		os.chdir( curDir )

	def GetWarnings(self):
		return self.WarningsFromFile()

	def GetOutputFile(self):
		return self.directory+"/" + self.outputFile

	def WarningsFromFile(self, inputDir=None, file=None, debug=None):
		if not inputDir: inputDir=self.directory
		if not file: file=self.outputFile
		# The file should be the outfile from above.
		reFileName = re.compile('^(\w+.\w+)\n')
		# match "$file($line) : $message" only at start of line.
		reLineWarn = re.compile('(^\S[^\(]+)\((\d+)\) : ')
		# find if it's an error or a warning
		reWarnSign = re.compile('^[^:]*warning')
		if inputDir is None: inputDir = "."
		if file is None: file = self.outputFile
		try:
			ERR = open( "%s/%s" % (inputDir, file) )
		except:
			return None

		warnings = []
		w = Warning()
		line = ERR.readline()
		curFile = None
		while line:
			if debug: print line,
			match = reLineWarn.search(line,0)
			if match:
				if debug: print ":matched"
				if w.message: warnings.append(w)
				w = Warning()
				matches = match.groups()
				w.file = matches[0]
				if curFile: w.sourceFile = curFile
				w.line = string.atoi(matches[1])
				w.message = string.strip(line[match.end():])
				wsMatch = reWarnSign.search(w.message,0)
				if wsMatch: w.type = 'warning'
				else: w.type = 'error'
			else:
				fileMatch = reFileName.search(line,0)
				if fileMatch:
					if w.message: warnings.append(w)
					w = Warning()
					curFile = fileMatch.groups()[0]
				elif line[0] is " " and w.message:
					w.message=w.message+line
					if debug: print ':appending'
				else:
					if w.message: warnings.append(w)
					w = Warning()
					if debug: print ':ignoring'
			line = ERR.readline()
		if w.message: warnings.append(w)
		return warnings

	def _OutputFileFromTarget(self,target):
		outputFile = "%s_out.txt" % (target)
		outputFile = outputFile.replace(' ', '_')
		self.outputFile = outputFile.replace('-', '_' )

