from MD2D import * #must be changed based on which dimension....

#from math import *
from Numeric import *
from exgcd import commonMillerIndices

class BicrystalInitializer:
	""" Initializes a bicrystal with the grain boundary plane orthogonal to the
	x axis. The orientation of the crystal surfaces is given by the miller indices .
	"""
	def __init__(self, lattice1, size1, xsurface1,  xsurface2, lattice2 = None, \
				 size2=None, ysurface1 = None, \
				 ysurface2 = None, boundaryConditions=None, overlap = None, \
				 prunerParameter = None, constrLayerWidth = 0):
		self.lattice1 = lattice1
		if lattice2 is None: self.lattice2 = lattice1
		else: self.lattice2 = lattice2
		self.size1 = size1 
		if size2 is None: self.size2 = size1[:]  # copy the values
		else :self.size2 = size2
		print self.size2
		self.xsurface1 = xsurface1
		self.xsurface2 = xsurface2
		self.ysurface1 = ysurface1
		self.ysurface2 = ysurface2
		self.boundaryConditions = boundaryConditions
		if overlap is not None: self.overlap = overlap
		else : self.overlap = 0.5*lattice1.GetLatticeSpacing() # the overlapping region
		if prunerParameter is not None: self.prunerParameter = prunerParameter
		else: self.prunerParameter = 0.7*lattice1.GetMinInteratomicSpacing()
		self.constrLayerWidth = constrLayerWidth
			
	def Create(self, atoms):
		rotation1, rectSize1 = self._CalculateGrainParameters(self.size1, self.xsurface1, self.lattice1, self.ysurface1)
		rotation2, rectSize2 = self._CalculateGrainParameters(self.size2, self.xsurface2, self.lattice2, self.ysurface2)		

# declare the rectangular cluster initializer
		rectangle = RectangularClusterInitializer(rectSize1, self.lattice1)
		# build the first crystal
		rectCenter1 = zeros(len(rectSize1), Float)
		rectCenter1[0] = -(rectSize1[0] - self.overlap)/2.
		rectSize1[0] = rectSize1[0] + self.overlap
		grain1 = DynamicListOfAtoms();
		self._CreateCrystal(rectangle, self.lattice1, rotation1, rectSize1, rectCenter1, grain1)
		# build the constrainted zone
		rectCenter1[0] = -rectSize1[0] + self.overlap - self.constrLayerWidth/2.
		rectSize1[0] = self.constrLayerWidth
		self.constrLOA1 = DynamicListOfAtoms();
		self._CreateCrystal(rectangle, self.lattice1, rotation1, rectSize1, rectCenter1, self.constrLOA1)
		# now build the second crystal
		rectangle.SetBravaisLattice(self.lattice2)
		rectCenter2 = zeros(len(rectSize1), Float)
		rectCenter2[0] =  (rectSize2[0] - self.overlap)/2.
		rectSize2[0] = rectSize2[0] + self.overlap
		grain2 = DynamicListOfAtoms()
		self._CreateCrystal(rectangle, self.lattice2, rotation2, rectSize2, rectCenter2, grain2)
		# build the constrainted zone
		rectCenter2[0] = rectSize2[0] - self.overlap + self.constrLayerWidth/2.
		rectSize2[0] = self.constrLayerWidth
		self.constrLOA2 = DynamicListOfAtoms();
		self._CreateCrystal(rectangle, self.lattice2, rotation2, rectSize2, rectCenter2, self.constrLOA2)
		# if we passed periodic boundary conditions, check that we can set a box size that is compatible with both lattices
		if isinstance(self.boundaryConditions, PeriodicBoundaryConditions):
			boxSize = zeros(len(rectSize1), Float) + 1.e8
			for i in range(1, len(rectSize1)): # the x direction is irrelevant.
				print 'ttt', rectSize1[i], rectSize2[i]
				if (abs(rectSize1[i] - rectSize2[i])/(rectSize1[i]+rectSize2[i])) < 1.0e-5 : boxSize[i]=rectSize1[i] 
			self.boundaryConditions.SetLength(boxSize)
			print 'bc Size = ', boxSize
		atoms.Merge(grain1)
		atoms.Merge(grain2)
		del grain1, grain2
		print 'removing overlap', atoms.GetNumber()
		OverlapPruner(self.prunerParameter).Transform(atoms)
		print 'done', atoms.GetNumber()

	def _CalculateGrainParameters(self, size, xsurface, lattice, ysurface=None): # returns the rotation matrix and the rectangle size
		rectangleSize = size
		latSpacing = lattice.GetLatticeSpacing()
		# we have to set the grain size such that it can accomodate the periodic boundary conditions
		# along y and z (or z alone).			
		if len(size) == 2: # first the 2D case
			k,l = xsurface
			if isinstance(lattice, TriangularLattice):
				# TRIANGULAR LATTICE    
				tmp = 2*sqrt(k*k+l*l-k*l)
				rotation = array([[sqrt(3)*k/tmp, (2*l-k)/tmp], [(k-2*l)/tmp, sqrt(3)*k/tmp]])
				# in the lattice's unit the periodicity arrays are [-l, k] along x and [2k-l,2l-k] along y
				rectangleSize = [sqrt(3)*tmp/2*latSpacing, tmp/2*latSpacing]
			if isinstance(lattice, SquareLattice):
				# 2D SQUARE LATTICE
				tmp = sqrt(k*k+l*l)
				rotation = array([[k/tmp,l/tmp],[-l/tmp, k/tmp]])
				rectangleSize = [latSpacing*tmp, latSpacing*tmp]
		if len(size) == 3: # 3D case
			if not isinstance(lattice, CubicLattice): raise TypeError, "need a cubic lattice"
			# 3D CUBIC LATTICE
			[kx,lx,mx] = xsurface
			if ysurface is None: [ky, ly, my] = [-lx, kx, 0]
			else:
				[ky, ly, my] = ysurface
				if abs(kx*ky+lx*ly+mx*my) > 1.0e-7: raise Exception, "xsurface is not orthogonal to ysurface" 
			tmpx = sqrt(kx**2+lx**2+mx**2) ; tmpy = sqrt(ky**2+ly**2+my**2);
			kz = lx*my-ly*mx; lz = -kx*my+mx*ky; mz = kx*ly-lx*ky
			cDenom = commonMillerIndices(kz,lz,mz)
			print "common Denominator", cDenom
			kz = kz / cDenom; lz = lz / cDenom; mz = mz / cDenom 
			tmpz = sqrt(kz**2+lz**2+mz**2)
			rotation = array([ [kx/tmpx, lx/tmpx, mx/tmpx], [ky/tmpy, ly/tmpy, my/tmpy],
												 [kz/tmpz, lz/tmpz, mz/tmpz]])
			rectangleSize = [latSpacing*tmpx, latSpacing*tmpy, latSpacing*tmpz]
		if isinstance(self.boundaryConditions, PeriodicBoundaryConditions):
			for i in range(1, len(rectangleSize)):
				tmp = rectangleSize[i]
				while rectangleSize[i] < size[i]: rectangleSize[i] = rectangleSize[i] + tmp
			rectangleSize[0] = size[0] # because we do not need periodic bc on the x axis
		else:
			rectangleSize = size
		return rotation, rectangleSize
			
	def _CreateCrystal(self, rect, lattice, rotation, size, center, crystal):
		rect.SetCenter(center)
		rect.SetRectangleSize(size)
		lattice.ResetOrientation()
		lattice.RotateLatticeVectors(rotation)
		rect.Create(crystal) 

