import sys
# Unix
sys.path.append('../../src/MD/SWIGBuild/i686/obj3D')
# Windows
sys.path.append('../../src/MD/SWIGBuild/')


#  This demonstration Python script runs a 3D simulation of Silicon using the
#  EDIP potential and periodic boundary conditions. Parameters such as time
#  step are listed at the top of the function and are mostly self explanatory.

#  More fundamental changes are to be made as follows: the value of DIMENSION
#  is set in the makefile. Be sure to change the if statement at the top of 
#  main to be consistent with the DIMENSION, and be sure that the potential
#  allows the chosen dimension. To change the potential simply change the name
#  of the class used to create the Potential object. If the lattice type and
#  spacing of the new potential are available, change these as well. This is
#  not necessary really for just checking energy conservation, which should 
#  hold regardless of configuration, but if the lattice spacing is much smaller
#  than the actual ground state spacing, then atoms may be too close for the 
#  time step and go unstable


from MD3D import *
from Numeric import *

# plotting
plotting = 1
plotRadius = 15
# time steps
dt = 0.01
nMajorSteps = 100
nMinorSteps = 10
# thermalization
temperature = 0.025
friction = 0.1
# geometry
latticeConst = 5.430
superCellLength = 3 * latticeConst
lengths = array([superCellLength,superCellLength,superCellLength])

# create ListOfAtoms, Potential, BoundaryConditions and NeighborLocator
atoms = DynamicListOfAtoms()
bc = SimplePeriodicBoundaryConditions()
bc.SetLength(lengths)
potential = SiliconEDIPPotential();
#StillingerWeberPotential()

neighborLocator = SimpleNeighborLocator(potential.GetCutoffDistance());  

atoms.SetBoundaryConditions(bc)
atoms.SetNeighborLocator(neighborLocator)


# create movers
verlet = VerletAtomsMover()
verlet.SetDt(dt)
verlet.SetNSteps(nMinorSteps)
verlet.SetPotential(potential)

langevin = LangevinAtomsMover()
langevin.SetDt(dt)
langevin.SetTemperature(temperature)
langevin.SetPotential(potential)
langevin.SetFriction(friction)
langevin.SetNSteps(100)

# create lattice and initialize the ListOfAtoms
# here it's a diamond lattice for Silicon
lattice = DiamondLattice(latticeConst)
cluster = RectangularClusterInitializer(lengths,lattice)
clusterCenter = lengths/2.
cluster.SetCenter(clusterCenter)
cluster.Create(atoms)

# the following is useful---to be able to subtract off energies later when calculating averages (helps round-off error)
groundEnergy = potential.CalculateEnergy(atoms);
# thermalize (otherwise the atoms will just stay in their ground state positions)
langevin.Move(atoms)


#add the plotting observer
if plotting:
    sys.path.append('Observers')
    from RasMolObserver import RasMolObserver3D
    #plot = RasMolObserver3D(radius = plotRadius*10)
    #atoms.AddObserver(plot)
    plot = PlotAtomsObserver(0,lengths[0], 0,lengths[1], plotRadius)
    atoms.AddObserver(plot)

# main loop
aveEnergy = 0.
aveEnergySq = 0.

for i in xrange(nMajorSteps):
    verlet.Move(atoms)
    atoms.Notify()
    kinetic = atoms.GetKineticEnergy()
    PE = potential.CalculateEnergy(atoms)
    totalEnergy = kinetic + PE
    print 'KE: ', kinetic, 'PE: ', PE, 'total: ', totalEnergy
    aveEnergy = aveEnergy + totalEnergy - groundEnergy
    aveEnergySq = aveEnergySq + (totalEnergy - groundEnergy) * (totalEnergy - groundEnergy)
    
aveEnergy = aveEnergy/nMajorSteps;
aveEnergySq = aveEnergySq/nMajorSteps;
print 'Average energy is ', groundEnergy + aveEnergy , '; variance is ' , aveEnergySq - aveEnergy * aveEnergy
