import os
import string
import tempfile

if os.name == 'nt' :
    import win32ui
#    import win32api
    import dde
    import time
else: from popen2 import popen2

class RasMolWinProcess:
    """This class creates the rasmol process and defines the __call__ functions
    that specifies how to send commands to raswin
    """
    def __init__(self):
        try: # we first look for RasMol in C:\Program Files
            self._myprocess = os.spawnv(os.P_NOWAIT, "C:\\Progra~1\\RasWin\RasWin", ())
        except: # if we didn't find it there, try to find it elsewhere
            (filename, i) = win32api.SearchPath(None, "RasWin", ".exe")
            self._myprocess = os.spawnv(os.P_NOWAIT, filename, ())
        self._server = dde.CreateServer()        
        self._server.Create("test")
        self._conversation = dde.CreateConversation(self._server)
        time.sleep(1)
        # it's necessary to wait a moment before we can connect to the server. Waiting
        # before the first attempt allows us to connect to the last created RasWin
        # (hopefully 1 sec is enough)
        for i in range(10):
            try:
                self._conversation.ConnectTo("RasWin","RemoteControl")
                break
            except:
                time.sleep(1)

    def __call__(self, command=""):
        """ send a command to the associated RasWin process """
        for i in range(30):
            try:
                self._conversation.Exec(command)
                break
            except:
                time.sleep(1)
               
class RasMolPipes:
    def __init__(self):
        (self._out_,self._in_)=popen2('rasmol ')
        
    def __call__(self, command=""):
        """Send a command string to RasMol.
        '__call__(command=s)': send the string s as a command to rasmol,
        followed by a newline and flush.  All interaction with the
        rasmol process is through this method and the filenames given
        to Update.
        """
        self._in_.write("\n"+command + "\necho done \n")
        self._in_.flush()
# wait for command to complete or EOF(in case command was quit)
        a=self._out_.readline() 
        while a!="done \n" and a != "\n":
            a=self._out_.readline()

    def __del__(self):
        self._in_.close()

class RasMolInterface:
    def __init__(self,filename=None,options=""):
        if os.name == 'nt': self._process = RasMolWinProcess()
        else : self._process = RasMolPipes()
        # os is deleted before our module, so we must keep references
        # to functions in os we need in the __del__ method. See comment in RasMolObserver
        self._unlink = os.unlink
        self._tmpscriptfilename_=tempfile.mktemp()
        if filename != None:
            self.Load(filename)

    def __del__(self):
        self('quit')
        del self._process
        try:
            self._unlink(self._tmpscriptfilename_)
        except:
            pass # if we never updated, the file may not exist.

    def Reload(self):
        self("write script '"+self._tmpscriptfilename_+"'")
        self.Script(self._tmpscriptfilename_)

    def Load(self,filename):
        self("zap") #empty rasmol's current data
        self("load '"+filename+"'")

    def __call__(self, command=""):
        self._process(command) #forward the command to the process

    def Script(self,filename):
        self("script '"+filename+"'")

    def Hardcopy(self,filename, format = 'gif'):
        """ saves a postscript file of the current ouput in filename.
        """
        self("write " + format + " '"+filename+"'")

    def ColorByClasses(self, colors):
        """Color the atoms according to their class.

        The colors argument is a dictionary with class numbers as
        keys, and colors as values.  A color is either a string recognized
        by RasMol as a color, or a 3-tuple containing RBG values (0-255).

        Using this method may interfere with settings made on the menus.
        """
        self("spacefill off")  # Avoid flickering image
        for k in colors.keys():
            self("select "+str(k))
            v = colors[k]
            if type(v) == type(""):
                self("colour "+v)
            elif len(v) == 3:
                self("colour [%d,%d,%d]" % v)
            else:
                raise ValueError, "Malformed color specification"
        self("select all")
        self("spacefill")
        
    def GetRotation(self):
        """Return the current rotation parameters.

        The rotation parameters are returned as a list containing the
        x, y and z rotation parameters in degrees.  Using these parameters,
        the present rotation can be obtained by rotating first around the
        z axis, then around the y axis and finally around the x axis.
        """
        tmpfile = tempfile.mktemp()
        self("write script '"+tmpfile+"'")
        f = open(tmpfile)
        data = f.readlines()  # Read the entire file
        f.close()
        os.unlink(tmpfile)
        rot = {}
        for line in data:
            if line[:7] == "rotate ":
                rot[line[7:8]] = int(line[9:]) # RasMol uses integers
        self._rot = rot
        if len(rot) == 3:
            return [rot["x"], rot["y"], rot["z"]]
        else:
            # If the system has not been rotated, no rotation info is
            # written to the script file
            return [0,0,0]
        

    def SetRotation(self, angles):
        """Set the RasMol rotation parameters.

        The rotation is specified by three integers giving the rotation
        about the x, y and z axes.  The rotations are applied in the
        reverse order, i.e. first around the z axis.
        """

        self("reset")
        self("rotate z "+str(angles[2]))
        self("rotate y "+str(angles[1]))
        self("rotate x "+str(angles[0]))
        
