# This is a simple application.

import sys
# Unix
sys.path.append('../../src/MD/SWIGBuild/i686/obj3D')
# Windows
sys.path.append('../../src/MD/SWIGBuild/')

from MD3D import *

# this is a simple script for a few Verlet steps with three atoms.

# Three atoms
atoms = DynamicListOfAtoms(3)

# Set positions and velocities. (Both are zero at creation time)
atoms.SetCartesianPosition(0,0,0.8)
atoms.SetCartesianPosition(1,1,1.0)
atoms.SetCartesianPosition(2,2,1.0)
atoms.SetCartesianVelocity(0,1,0.3)
atoms.SetCartesianVelocity(2,1,-0.3)

# Lennard Jones potential, cut off smoothly at 2.7 units
potential = CutLennardJonesPotential()
cutoff = potential.GetCutoffDistance()

# neighborLocator will give atoms up to cutoff (default noNeighborLocator)
neighborLocator = SimpleNeighborLocator(cutoff)
atoms.SetNeighborLocator(neighborLocator)

# Time step given by simple Verlet algorithm
# Warning: our version of Verlet isn't standard!
mover = VerletAtomsMover(potential, 0.01, 10)

# Add graphics
if "norasmol" not in sys.argv:
	sys.path.append('Observers')
	from RasMolObserver import RasMolObserver3D
	ras = RasMolObserver3D(radius=100)
	atoms.AddObserver(ras)

# 200 time steps: 20 groups of ten, runs to 200*0.01 = 2 time units
for i in range(200):
    atoms.Notify()
    PE = potential.CalculateEnergy(atoms)
    KE = atoms.GetKineticEnergy()
    print PE + KE
    mover.Move(atoms)
    
