// BinaryReader.cpp: implementation of the BinaryReader class.
//
//////////////////////////////////////////////////////////////////////


#include "DigitalMaterial.h"
#include "BinaryReader.h"
#include "DMException.h"
#include "Serializable.h"
#include "SerialFactory.h"
#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BinaryReader::BinaryReader(const char *buffer): DMReader("")
{
    this->buffer      = buffer;
    this->currentPos  = 0;
}

BinaryReader::~BinaryReader()
{
}

void BinaryReader::SetBuffer(const char *buffer)
{
    this->buffer = buffer;
    this->currentPos = 0;
}


std::string BinaryReader::GetType(const string &name)
{
    throw DMException("BinaryReader", "GetType", 
        "Types are not put/read in binary buffers");
}

Serializable *BinaryReader::Get(const string &name)
{
    Serializable *serializable = SerialFactory::Instance()->New(GetString(""));
    serializable->Load(this);
    return serializable;
}



bool BinaryReader::Fill(Serializable *toFill, const string &name)
{
    throw DMException("BinaryReader", "Fill", "Not Implemented yet");
}

bool BinaryReader::GetBool( const string& name )
{
    int tmp = GetInt("");
    return (tmp != 0);
}

int BinaryReader::GetInt(const string &name)
{
    const int size = sizeof(int);
    const int *result = reinterpret_cast<const int *> (buffer + currentPos);
    currentPos += size;
    return *result;
}

double BinaryReader::GetDouble(const string &name)
{
    const int size = sizeof(double);
    const double *result = reinterpret_cast<const double *> (buffer + currentPos);
    currentPos += size;
    return *result;
}

string BinaryReader::GetString(const string &name)
{
    string result = buffer+currentPos;
    currentPos += result.size() + 1;
    return result;
}

int *BinaryReader::GetIntArray(vector<int> &dims, const string &name)
{
    dims.clear();
    const int nDims = GetInt("");
    int tot = 1;
    for(int i=0; i<nDims; ++i)
    {
        int tmp = GetInt("");
        dims.push_back(tmp);
        tot *= tmp;
    }
    int *data = new int[tot];
    memcpy(data, buffer+currentPos, tot*sizeof(int));
    currentPos += tot*sizeof(int);
    return data;	
}

double *BinaryReader::GetDoubleArray(vector<int> &dims, const string &name)
{
    dims.clear();
    const int nDims = GetInt("");
    int tot = 1;
    for(int i=0; i<nDims; ++i)
    {
        int tmp = GetInt("");
        dims.push_back(tmp);
        tot *= tmp;
    }
    double *data = new double[tot];
    memcpy(data, buffer+currentPos, tot*sizeof(double));
    currentPos += tot*sizeof(double);
    return data;	
}
