// DMWriter.h: interface for the DMWriter class.
//
//////////////////////////////////////////////////////////////////////

#ifndef DMWRITER_H
#define DMWRITER_H

#include "DMFirstHeader.h"
#include <string>
#include <vector>
#include "DigitalMaterial.h"
#include "DMException.h"
#include "Serializable.h"

class DMWriter
{
public:
    DMWriter(const std::string &filename) : filename(filename) {}
    virtual ~DMWriter() {}
    virtual void Open() = 0;
    virtual void Close() = 0;
    virtual void Put(const Serializable *serializable, const std::string &name="root") = 0;
    virtual void PutBool( bool b, const std::string& name ) = 0;
    virtual void PutInt(int n, const std::string &name) = 0;
    virtual void PutDouble(double x, const std::string &name) = 0;
    virtual void PutString(const std::string &str, const std::string &name) = 0;
    virtual void PutIntArray(const int *data, const std::vector<int> &dims, const std::string &name) = 0;
    virtual void PutDoubleArray(const double *data, const std::vector<int> &dims, const std::string &name) = 0;

    template<class Type> 
        void PutArray(const Type *data, std::vector<int> &dim, const std::string &name)
    {
        throw DMException("DMWriter", "PutArray<>", "This type not implemented");
    }
    void PutArray(const int *data   , std::vector<int> &dims, const std::string &name)
        { PutIntArray(data, dims, name);}
    void PutArray(const double *data, std::vector<int> &dims, const std::string &name)
        { PutDoubleArray(data, dims, name);}

protected:
    std::string filename;
};



#endif // DMWRITER_H
