// NetCDFWriter.h: interface for the netCDFWriter class.
//
//////////////////////////////////////////////////////////////////////

#ifndef NETCDFWRITER_H
#define NETCDFWRITER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "DMWriter.h"
#include "NetCDFStructure.h"


// XXX This uses the C interface to netCDF. 
// There is a C++ interface, which keeps track of the ncxxxID numbers and such.
// The binaries aren't available for Windows, though, and somewhere in the
// documentation it suggests that the C++ interface isn't as well used and debugged
// This does seem pretty ugly, though: we should explore using C++ more effectively

//! Writes serial data to disk using the Unidata NetCDF format.
class NetCDFWriter: public DMWriter
{
	bool bFileOpen; //!< Whether we have a currently open file.
	NetCDFStructure::PathType path; //!< Represents virtual directory structure.
	int ncID; //!< NetCDF ID of the open data file.
public:
	//! Creates a new file.
	NetCDFWriter(const std::string &filename = "data.nc");
	virtual ~NetCDFWriter();

	//! Open the file you created.
	void Open();
	//! Close the open file.
	void Close();

	//! Store a serializable object with a given name.
	void Put(const Serializable *serializable, const std::string &name="root");

	void PutBool(bool b, const std::string &name);
	void PutInt(int n, const std::string &name);
	void PutDouble(double x, const std::string &name);
	void PutString(const std::string &str, const std::string &name);
	void PutIntArray(const int *data, const std::vector<int> &dims,
		const std::string &name);
	void PutDoubleArray(const double *data, const std::vector<int> &dims,
		const std::string &name);

	int GetMaxNumDimensions() const;
private:
	static void BeginDefinition( int ncID );
	static void EndDefinition( int ncID );
	static int DefineVariable( int ncID, const std::string& sItemName,
		int type );
	static int DefineDimension( int ncID, const std::string& sDimItem, int size );
	static int DefineArray( int ncID, const std::vector<int>& dims,
		const std::string sItem, int type );
	void WriteZeroDimensions( const std::vector<int>& dims,
		const std::string& sItemName );
	static size_t DimsToSize( const std::vector<int>& dims );
	static bool CheckDims( bool isNonZeroData, const std::vector<int>& dims,
		const std::string& name );
};


#endif //NETCDFWRITER
