// Buffer.cpp: implementation of the Buffer class.
//
//////////////////////////////////////////////////////////////////////
#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#ifdef DMPARALLEL
#include "MPISendBuffer.h"

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MPISendBuffer::MPISendBuffer(const MPI_Comm comm): comm(comm)
{
}

MPISendBuffer::~MPISendBuffer()
{
}


void MPISendBuffer::PutBool( bool b, const string& name )
{
    int need;
    MPI_Pack_size(1, MPI_INT, comm, &need);
    if(need + currentSize > bufferSize)
        Allocate(bufferSize + 5*need);
    int intBool = (int)b;
    MPI_Pack(&intBool, 1, MPI_INT, buffer, bufferSize, &currentSize, comm);
}

void MPISendBuffer::PutInt(int n, const string &name)
{
    int need;
    MPI_Pack_size(1, MPI_INT, comm, &need);
    if(need + currentSize > bufferSize)
        Allocate(bufferSize + 5*need);
    MPI_Pack(&n, 1, MPI_INT, buffer, bufferSize, &currentSize, comm);
}

void MPISendBuffer::PutDouble(double x, const string &name)
{
    int need;
    MPI_Pack_size(1, MPI_DOUBLE, comm, &need);
    if(need + currentSize > bufferSize)
        Allocate(bufferSize + 5*need);
    MPI_Pack(&x, 1, MPI_DOUBLE, buffer, bufferSize, &currentSize, comm);
}

void MPISendBuffer::PutString(const string &str, const string &name)
{
    int strSize = str.size();
    PutInt(strSize,"");
    int need;
    MPI_Pack_size(strSize, MPI_CHAR, comm, &need);
    if(need + currentSize > bufferSize)
        Allocate(bufferSize + 5*need);
    MPI_Pack((void *)(str.c_str()), strSize, MPI_CHAR, buffer, bufferSize, &currentSize, comm);
}

void MPISendBuffer::PutIntArray(const int *data,
                                const vector<int> &dims, 
                                const string &name)
{
    int nDims = dims.size();
    int tot = 1;
    for(int ii=0; ii<nDims; ++ii) tot *= dims[ii];
    int need;
    MPI_Pack_size(1+nDims+tot, MPI_INT, comm, &need);
    if(need + currentSize > bufferSize) Allocate(bufferSize + need);
    
    MPI_Pack(&nDims, 1, MPI_INT, 
        buffer, bufferSize, &currentSize, comm);
    MPI_Pack((void *)&dims[0], dims.size(), MPI_INT, 
        buffer, bufferSize, &currentSize, comm);
    MPI_Pack((void *)data, tot, MPI_INT, buffer, bufferSize, &currentSize, comm);
}

void MPISendBuffer::PutDoubleArray(const double *data,
                                   const vector<int> &dims, 
                                   const string &name)
{
    int nDims = dims.size();
    int tot = 1;
    for(int ii=0; ii<nDims; ++ii) tot *= dims[ii];
    int need1;
    int resMPI;
    resMPI = MPI_Pack_size(1+nDims, MPI_INT, comm, &need1);
    ASSERT(resMPI == MPI_SUCCESS);
    int need2;
    resMPI = MPI_Pack_size(tot, MPI_DOUBLE, comm, &need2);
    ASSERT(resMPI == MPI_SUCCESS);
    if(need1 + need2 + currentSize > bufferSize) Allocate(bufferSize + need1 + need2);
    
    resMPI = MPI_Pack(&nDims, 1, MPI_INT, 
        buffer, bufferSize, &currentSize, comm);
    ASSERT(resMPI == MPI_SUCCESS);
    resMPI = MPI_Pack((void *)&dims[0], dims.size(), MPI_INT, 
        buffer, bufferSize, &currentSize, comm);
    ASSERT(resMPI == MPI_SUCCESS);
    resMPI = MPI_Pack((void *)data, tot, MPI_DOUBLE, buffer, bufferSize, &currentSize, comm);
    ASSERT(resMPI == MPI_SUCCESS);
}

#endif //DMPARALLEL
