// ProtocolFactory.cpp: implementation of the ProtocolFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "ProtocolFactory.h"

#include "LocalProtocol.h"

#ifdef DMPARALLEL
#include "Parallel/MPIProtocol.h"
#endif //DMPARALLEL


using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ProtocolFactory *ProtocolFactory::_instance = 0;

DMProtocol *dmProtocol = 0;

// make sure the factory is instantiated
ProtocolFactory *ProtocolFactorySingleton = ProtocolFactory::Instance();


ProtocolFactory::ProtocolFactory()
{
    dmProtocol = new LocalProtocol();
#ifdef DMPARALLEL
    int doneInit;
    int res = MPI_Initialized(&doneInit);
    if (doneInit) SetProtocol(World); 
#endif // DMPARALLEL
}

ProtocolFactory::~ProtocolFactory()
{
    while(!protocolStack.empty())
    {
        delete protocolStack.top();
        protocolStack.pop();
    }
}

ProtocolFactory* ProtocolFactory::Instance()
{
    if(0 == _instance) _instance = new ProtocolFactory();
    return _instance;
}


void ProtocolFactory::SetProtocol(Comm comm)
{
    protocolStack.push(dmProtocol);
    if(comm == World) 
{
#ifdef DMPARALLEL
        dmProtocol = new MPIProtocol();
#else
        throw DMException("ProtocolFactory","SetProtocol",
                          "asked for MPI, but compiled without DMPARALLEL flag");
#endif // DMPARALLEL
}
    else dmProtocol = new LocalProtocol();
}

void ProtocolFactory::Reset()
{
    delete dmProtocol;
    dmProtocol = protocolStack.top();
    protocolStack.pop();
}
