#include "DMFirstHeader.h"
#include <vector>
#include <algorithm>

#include "Rand.h"
#include "DMWriter.h"
#include "DMReader.h"
#include "SerialBuilder.h"
#include "DMLastHeader.h"

const std::string sType = std::string("Rand");
const SerialBuilder<Rand> *_creator = new 
	SerialBuilder<Rand>(sType);

const double Rand::cd=7654321.0/16777216.0;
const double Rand::cm=16777213.0/16777216.0;

Rand::Rand(int seed) {
	this->iOriginalSeed = seed;
	ranmarin(abs(seed%BIGPRIME));
	outputReady=false;
}

Rand::Rand() {
	this->iOriginalSeed = 1;
	ranmarin(1);
	outputReady=false;
}

void Rand::Save( DMWriter* writer ) const
{
	std::vector<int> shape;
	int iOutput;
	iOutput = (outputReady) ? 1 : 0;
	writer->PutInt(iOutput, "isOutputReady" );
	writer->PutInt( i97, "i97" );
	writer->PutInt( j97, "j97" );
	writer->PutDouble( c, "c" );
	writer->PutDouble( cd, "cd" );
	writer->PutDouble( cm, "cm" );
	shape.push_back(97);
	writer->PutDoubleArray( u, shape, "u97" );
}

void Rand::Load( DMReader* reader)
{
	std::vector<int> shape;
	double *dTemp;
	int iOutput = reader->GetInt( "isOutputReady" );
	outputReady = (iOutput == 1);
	i97 = reader->GetInt( "i97" );
	j97 = reader->GetInt( "j97" );
	c = reader->GetDouble( "c" );
	double trash = reader->GetDouble( "cd" );
	trash = reader->GetDouble( "cm" );
	dTemp = reader->GetDoubleArray( shape, "u97" );
	std::copy(dTemp,dTemp+97, (double*) u);
	delete[] dTemp;
}

const std::string &Rand::GetType() const
{
	return sType;
}
