#ifndef _RUNNABLE_H_
#define _RUNNABLE_H_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif

//! A class inherits from this if it would like to run in a thread.
/**
 * This is just like the Java Runnable class.  A class inherited from
 * Runnable can be run in a thread until the class exits from its 
 * Run() method.  There is no graceful way to stop, pause, or resume
 * a thread created for a Runnable class.  The best way to use it is
 * for something that can just run to completion.
 *
 * \code
 * class MyCalc : public Runnable {
 *     int i;
 *     void Run() {
 *         while ( i < 20 ) {
 *             i = i + 1; // Do calculation here
 *             // Then notify display you have done a step.
 *         }
 *     }
 * };
 *
 * int main(void) {
 *     MyCalc sim();
 *     Thread lightning(&sim);
 *     lightning.Start();
 *     DoUserInterface();
 * }
 * \endcode
 * \ingroup dmsystem
 * \author Drew Dolgert
 * \date 21 March 2000
 * \sa Threadable, Thread
 */
class Runnable
{
public:
	virtual void Run() = 0;
};

#endif //_RUNNABLE_H_
