/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = r + a * b + c * d + e
 */

#include "vectools.h"

/* res[] += a[] * b + c[] * d + e[] */
void vec_self_dbl_mul_scal_add_add(double *res, const double *a, double b,
				   const double *c, double d, const double *e, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, *c, d, *e)
#endif
    
    int i,limit;

    double a0, a1, a2, a3, a4, a5, a6, a7;
    double c0, c1, c2, c3, c4, c5, c6, c7;
    double e0, e1, e2, e3, e4, e5, e6, e7;
    double s0, s1, s2, s3, s4, s5, s6, s7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	s3 = res[i+3];
	s4 = res[i+4];
	s5 = res[i+5];
	s6 = res[i+6];
	s7 = res[i+7];
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	c4 = c[i+4];
	c5 = c[i+5];
	c6 = c[i+6];
	c7 = c[i+7];
	e0 = e[i];
	e1 = e[i+1];
	e2 = e[i+2];
	e3 = e[i+3];
	e4 = e[i+4];
	e5 = e[i+5];
	e6 = e[i+6];
	e7 = e[i+7];
	/* The parentheses help the compiler to generate FMA instructions */
	res[i] = a0 * b + (c0 * d + e0) + s0;
	res[i+1] = a1 * b + (c1 * d + e1) + s1;
	res[i+2] = a2 * b + (c2 * d + e2) + s2;
	res[i+3] = a3 * b + (c3 * d + e3) + s3;
	res[i+4] = a4 * b + (c4 * d + e4) + s4;
	res[i+5] = a5 * b + (c5 * d + e5) + s5;
	res[i+6] = a6 * b + (c6 * d + e6) + s6;
	res[i+7] = a7 * b + (c7 * d + e7) + s7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	s0 = res[i];
	a0 = a[i];
	c0 = c[i];
	e0 = e[i];
	res[i] = a0 * b + (c0 * d + e0) + s0;
    }
}

/* res3[][0..2] += a3[][0..2] * b[offset[]] + c3[][0..2] * d[offset[]] +
                   e3[][0..2] */
void vec3_self_dbl_mul_indir_add_add(double (*res3)[3], const double (*a3)[3],
				     const double *b, const double (*c3)[3], const double *d,
				     const double (*e3)[3], const int *offset, int n)
{
    int i,limit;
    double *res, *a, *c, *e;
    double a0, a1, a2;
    double c0, c1, c2;
    double e0, e1, e2;
    double s0, s1, s2;
    double bx, dx;
#ifdef AIX
#pragma disjoint (*res3, *a3, *b, *c3, *d, *e3)
#pragma disjoint (*res, *a, *b, *c, *d, *e)
#endif

    res = (double *) res3;
    a = (double *) a3;
    c = (double *) c3;
    e = (double *) e3;
    
    limit = 3*n;
    for (i = 0; i < limit; i+=3)
    {
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	e0 = e[i];
	e1 = e[i+1];
	e2 = e[i+2];
	bx = b[*offset];
	dx = d[*offset++];
	/* The parentheses help the compiler to generate FMA instructions */
	res[i] = a0 * bx + (c0 * dx + e0) + s0;
	res[i+1] = a1 * bx + (c1 * dx + e1) + s1;
	res[i+2] = a2 * bx + (c2 * dx + e2) + s2;
    }
}
