/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 *
 * Calculates an element of the outher product of a vector and itself:
 * r = a[ix] * a[iy]
 */

#include "vectools.h"

/* r[] = a[][ix] * a[][iy] */
void vec_outherprod(double *res, const double a[][3], int ix, int iy, int n)
{
    int i, limit;

    double a0, a1, a2, a3;
    double b0, b1, b2, b3;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i][ix];
	a1 = a[i+1][ix];
	a2 = a[i+2][ix];
	a3 = a[i+3][ix];
	b0 = a[i][iy];
	b1 = a[i+1][iy];
	b2 = a[i+2][iy];
	b3 = a[i+3][iy];
	res[i] = a0 * b0;
	res[i+1] = a1 * b1;
	res[i+2] = a2 * b2;
	res[i+3] = a3 * b3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i][ix];
	b0 = a[i][iy];
	res[i] = a0 * b0;
    }
}

	
