// SimpleMatrixInverter.h  ---  inverts small matrices (often DIMENSION x DIMENSION)

#ifndef SIMPLEMATRIXINVERTER_H
#define SIMPLEMATRIXINVERTER_H

#include "DigitalMaterial.h"
#include "SprintString.h"
#include "DMException.h"
#include "lapack.h"

template<int N>
class SimpleMatrixInverter
{
public:
	void Invert(double matrix[N][N])
		{
			int size = N;
			int ipiv[N];
			int info;
			double fMatrix[ N * N ];
			int lwork = N * N;
			double *work = new double[lwork];
			int index = 0;
			for(int idx2=0;idx2<N;++idx2)
				for(int idx=0;idx<N;++idx)
				{
					fMatrix[index++] = matrix[idx][idx2];
				}
			

			// LU factorize the fMatrix
			dgetrf_( &size, &size, fMatrix, &size,  ipiv,  &info );
			
			ASSERT(info == 0);
			if(info < 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetrf: %d-th argument had an illegal value", -info));
			if(info > 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetrf: U(%d,%d) is exactly zero", info,info));
			
			// compute the inverse of the matrix from the LU factorization
			dgetri_( &size, fMatrix, &size, ipiv, work, &lwork, &info );
			
			ASSERT(info == 0);
			if(info < 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetri: %d-th argument had an illegal value", -info));
			if(info > 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetri: U(%d,%d) is exactly zero", info,info));
			
			delete [] work;
			
			index = 0;
			for(int idx=0;idx<N;++idx)
				for(int idx2=0;idx2<N;++idx2)
					matrix[idx2][idx] = fMatrix[index++];
	
		}
};

#endif // SIMPLEMATRIXINVERTER_H
