// SteepestDescentMinimizer.h: interface for the SteepestDescentMinimizer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef STEEPESTDESCENTMINIMIZER_H
#define STEEPESTDESCENTMINIMIZER_H

#include "DMFirstHeader.h"
#include <math.h>
#include <string>
#include "DigitalMaterial.h"
#include "Minimizer.h"


//! Does steepest descent minimization,taking a fixed step each time. Does not do actual line minimization so does not make use of the ObjectiveFunction.



/**
 *
 *
 * \author nbailey 
 * \date 9/21/2000
 * \ingroup moleculardynamics
 */




class SteepestDescentMinimizer: public Minimizer  
{
public:
  SteepestDescentMinimizer(double tolerance, int maximumIterations,double stepParameter);
  virtual ~SteepestDescentMinimizer();
	double Minimize(double *parameters,Minimizable *minimizable);  
  double GetTolerance() const {return tolerance;}
  void SetTolerance(double tol) {tolerance = tol;}
  int GetNIterations() const {return nIterations;} // returns number of iterations taken during the last call to minimize
  double GetFinalGradNorm() const {return gradNorm;}
  int GetMaxIterations() const {return maximumIterations;}
  void SetMaxIterations(int maxIt) {maximumIterations = maxIt;}
  
  void SetStepParameter(double sp) {stepParameter = sp;}
  double GetStepParameter() const {return stepParameter;}
  void Verbose() {verbose = true;}
  void Quiet() {verbose = false;}
  
 protected:
  int maximumIterations;
  double tolerance;
  int nIterations;	
  double gradNorm;
  Minimizable *minimizable;
  bool verbose;
#ifndef SWIG
  
  double stepParameter;
    
  double *parameters;
#endif // SWIG
  
};



#endif // !defined(STEEPESTDESCENTMINIMIZER_H)
