/* UnixThread.h - 21 November 1999
 * Windows needs to put the simulation in a thread.
 * If we derive the simulation from a thread, we would
 * have trouble using the simulation class on different
 * platforms because threads are a hairy subject.
 * Therefore this thread will own a Threadable object.
 *
 * The rules of this thread class are as follows:
 * It is synchronous, so the methods do what they claim before they leave.
 * Only one client thread gets access at a time, so the thread is thread-safe.
 * Note no functions tell you how the thread is doing.  This is a thread class
 * for the decisive.
 */

#ifndef _UnixThread_
#define _UnixThread_ 1

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Threadable.h" // includes Runnable
#include "ThreadImp.h"

class UnixThread : public ThreadImp
{
public:
	// Pass by reference because UnixThread is not
	// responsible for threadable deletion.
	UnixThread();
	virtual void SetTask(Runnable *thread);
	virtual void SetTask(Threadable *thread);
	virtual int Start();
	virtual int Stop();
	virtual int Pause();
	virtual int Resume();
//	virtual void Lock();
//	virtual void Unlock();
	virtual int Toggle();
	// interface to simulation
	virtual bool IsRunning();
	virtual ~UnixThread();
};


#endif // _UnixThread_
