#ifndef _WinProcessFactory_H_
#define _WinProcessFactory_H_

#include "DigitalMaterial.h"
#include "ProcessFactory.h"

//! A concrete ProcessFactory for Windows.
/**
 * This is a concrete implementation of the abstract ProcessFactory.
 * In order to use ProcessFactory::Instance() in a program, you must
 * first instantiate a WinProcessFactory with the call
 * WinProcessFactory::Create().
 *
 * Note: Imp as in ThreadImp or RWLockImp mean implementation
 *
 * \author Drew Dolgert
 * \date 21 March 2000
 * \bug There is an easier way to instantiate the ProcessFactory.
 *		Because only the WinProcessFactory will be included in the
 *		build on a Windows machine (no reason to include UnixProcessFactory),
 *		we could have WinProcessFactory instantiate itself automatically
 *		using a static member variable.  Cute, but quiet and effective.
 * \sa ProcessFactory, UnixProcessFactory
 * \ingroup dmsystem
 */
class WinProcessFactory : public ProcessFactory {
public:
	static void Create() {
		ProcessFactory::_instance = new WinProcessFactory();
	}
	virtual ThreadImp* GetThread();
	virtual RWLockImp* GetRWLock();
	virtual EggTimer*  GetEggTimer();
protected:
	WinProcessFactory() {}
};

#endif // _WinProcessFactory_H_
