// lapack.h header file for the lapack declarations
//
////////////////////////////////////////////////


extern "C" 
{
	// LAPACK driver routines follow

	// routine to solve a general system of linear equations (simple driver)
#ifdef _WIN32
#	define dgesv_ DGESV
#endif
	int dgesv_(int *n, int *nrhs, double *a, int *lda, int *ipiv, double *b, int *ldb, int *info); 

	// routine to get eigenvalues and (left and right)eigenvectors of a general matrix.
	// char argument !!! special attention for windows (stilil to be done)
#ifdef _WIN32
#	define dgeev_ DGEEV
#endif
	int dgeev_(char* jobvl,char* jobvr,int* n, double* a, int* lda,double* wr,double* wi, double* vl, 
			   int* ldvl, double* vr, int* ldvr, double* work, int* lwork, int* info);

	// routine to get generalized eigenvalues and eigenvectors for a system of 2 symmetric matrices.
#ifdef _WIN32
#	define dsygv_ DSYGV
#endif
	int dsygv_(int *itype, char *jobz, char *uplo, int *n, double *a, int *lda, double *b, int *ldb, double *w, double *work, int *lwork, int *info );


	// routines for computing minimum norm solution to a possibly singular or degenerate linear system using singular value decomposition. dgelsd is faster than the older dgelss, but needs more workspace
#ifdef _WIN32
#	define dgelss_ DGELSS
#endif
	int dgelss_(int *m, int *n, int *nrhs, double *a, int *lda, double *b, int *ldb, double *s, double *rcond, int *rank, double *work, int *lwork, int *info);

#ifdef _WIN32
#	define dgelsd_ DGELSD
#endif
	int dgelsd_(int *m, int *n, int *nrhs, double *a, int *lda, double *b, int *ldb, double *s, double *rcond, int *rank, double *work, int *lwork, int *iwork, int *info);

	// routine for eigenvectors and eigenvalues of a symmetric matrix
#ifdef _WIN32
#	define dsyev_ DSYEV
#endif
	int dsyev_(char* jobz,char* uplo,int* n,double* fa,int* lda,double* w, double* work,int* lwork,int* info);


	// routine for solving a real symmetric indefinite system of linear equations AX=B (simple driver) 
#ifdef _WIN32
#	define dsysv_ DSYSV	
#endif
	int dsysv_(char* uplo,int* n,int *nrhs,double *a,int *lda,int *ipiv,double *b,int *ldb,double *work, int *lwork,int *info);


	// routine for doing LU factorization of general m x n matrix
#ifdef _WIN32
#	define dgetrf_ DGETRF
#endif
	int dgetrf_( int *m, int *n, double *a, int *lda, int *ipiv, int *info );

	// routine to compute the inverse of a matrix using the LU factorization provided by dgetrf
#ifdef _WIN32
#	define dgetri_ DGETRI
#endif
	int dgetri_( int *n, double *a, int *lda, int *ipiv, double *work, int *lwork, int *info );
}
