// MDMain.cpp : main loop for Unix, Windows Console applications

#include <iostream>
#include <fstream>
#include <math.h>
#include "MDMain.h"

#ifdef _DEBUG
// XXX #define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Main loop: sets up and controls the simulation
// The software is designed so that this can be done in an interpreted, 
// scripting language

void main(int argv, char **argc)
{

  EMTPotential potential("Cu");
  const double chemicalPotential = -potential.GetEnergyCurvatureScale()/2.*potential.GetLengthScale()*potential.GetLengthScale();

  FCCLattice lattice(potential.GetLengthScale()*sqrt(2.));
  double e0;
  RescaleLatticeToMinimizeEnergy(&potential, &lattice, e0);
  //  cout.precision(12);
  //  cout<< lattice.GetLatticeConstant() << " " << e0 << endl;
  //  exit(0);


  double shift[DIMENSION] = {0.11,0.17, 0.025}; // this is to avoid having atoms right on the boundaries of the clusters
  lattice.SetOrigin(shift);

  SimplePeriodicBoundaryConditions *bc = new SimplePeriodicBoundaryConditions;


  int orientation[DIMENSION] = {1,0,0};
  AtomicSurfaceInitializer surface(&lattice, orientation, bc);
  surface.SetDepthOfFixedLayer(potential.GetCutoffDistance());
  surface.SetDepthOfDynamicLayer(4.*potential.GetCutoffDistance());
  surface.SetMinimalLength(6);  // the size of the surface is at least 6 lattice spacings

  NeighborListPBC *NBList = new NeighborListPBC(potential.GetCutoffDistance(), potential.GetCutoffList());
  // NeighborListPBC *NBList = new NeighborListPBC(potential.GetCutoffDistance(), potential.GetCutoffList());

  std::ofstream out("Cuout.log",ios::app); 

  int k,l,m;
  double phi, theta; // orientation of the surface;
  for(m=0; m<=10; m++) for(k=1; k<=10; k++) 
    for(l=m; l<=k; l++) if(surface.SimplifiedVector(k,l,m))
    {
          cout << k << " " << l << " " << m << " ";
	  out <<  k << "\t" << l << "\t" << m << "\t";
	 
	  double norm1 = sqrt(k*k+l*l+m*m), norm2 = sqrt(k*k+l*l);
	  double length = 0.; 
	  while(length < 6.) length += sqrt(k*k+l*l)*sqrt(k*k+l*l+m*m);
	  length *= potential.GetLengthScale()*sqrt(2);
	  
	  double width = 0.; 
	  while(width  < 6.) width  += sqrt(k*k+l*l);
	  width  *= potential.GetLengthScale()*sqrt(2);   // we are dealing with a cubic lattice
	  double lengths[3] = {length, width, 1.0E99}; // not periodic along z
	  bc->SetLength(lengths);
	  
	  orientation[0] = k;
	  orientation[1] = l;
	  orientation[2] = m;

	  surface.SetOrientation(orientation);

	  ListOfTypedAtoms atoms;
	  atoms.SetBoundaryConditions(bc);
	  atoms.SetNeighborLocator(NBList);

	  surface.Create(&atoms);
	    
	  cout << atoms.ListOfAtoms::GetNumber() << " atoms, E =  " << potential.CalculateEnergy(&atoms) << endl;

	  //	  PlotAtomsObserver *plot = new PlotAtomsObserver(100.0, 30.0, 8);
	  //	  atoms.Attach(plot);
	  //	  atoms.Notify();
  

	  //	  HooverAtomsMover hoover(&potential,0.01, 0.03,1., 100);
	  QuickMinAtomsMover minimize(&potential,0.06,1.0e-5);
  	  minimize.SetMaxSteps(20);  

	  //	  atoms.Notify();

	  cout.precision(12);
	  out.precision(12);
	  double oldE, newE;
	  double eUndeformed = (potential.CalculateEnergy(&atoms) - 
				atoms.ListOfAtoms::GetNumber()*chemicalPotential)/2.;
	  out << atoms.ListOfAtoms::GetNumber() << "\t" << potential.CalculateEnergy(&atoms) << "\t"
	      << bc->GetLength(0) <<"\t" << bc->GetLength(1) ;
	  //	  for(int ii=1; ii<20; ii++)
	  // {
	      //	      hoover.Move(&atoms);
	      //	      atoms.Notify();
	      //	      cout << potential.CalculateEnergy(&atoms) + atoms.GetKineticEnergy() << endl;
	  //  }
	  for (int i=1; i<1000; i++)
	    {
	      //verlet.Move(atoms);
	      minimize.Move(&atoms);
		      //exit(0);
	      //	      atoms.Notify();
	      newE = (potential.CalculateEnergy(&atoms) - eUndeformed - 
		      atoms.ListOfAtoms::GetNumber()*chemicalPotential)/bc->GetLength(0)/bc->GetLength(1);
	      cout <<i<< " " <<newE<< endl;
	      if (fabs((newE-oldE)/oldE) < 1.0e-5) {
		//		cout << "fini" << endl;
		i=1000;
	      }
	      oldE = newE;
	    }
	  out << "\t" << newE<< endl;
	  //char filename[22];
	  //sprintf(filename,"SURF/%1.1i_%1.1i_%1.1i",k,l,m);
	  //	  atoms.SaveNetCDF(filename);
    }
  out.close();
}
