// CylindricalClusterInitializer.h: interface for the CylindricalClusterInitializer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CYLINDRICALCLUSTERINITIALIZER_H
#define CYLINDRICALCLUSTERINITIALIZER_H

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // SWIG


//! This will generate a cylindrical cluster

/*!
  This is a slight modification of SphericalClusterInitializer. You pass in a 
  radius a length and a direction. The radius refers perpendicular distance
  from the axis which is along the given direction.

  \author nbailey
  \date July 4, 2001
  \ingroup moleculardynamics

 */

#include "ClusterInitializer.h"

class CylindricalClusterInitializer : public ClusterInitializer
{
public:
	CylindricalClusterInitializer(double radius, double length, double direction[DIMENSION], BravaisLatticeWithBasis *bravais);
	virtual ~CylindricalClusterInitializer();
	void SetRadius(double radius); 
	double GetRadius() const {return radius;}
	
	void SetLength(double length);
	double GetLength() const {return length;}

	void SetDirection(double direction[DIMENSION]);
	const double *GetDirection() const {return direction;}	

protected:
	double radius;
	double length;
	double direction[DIMENSION];
	int SetMaxSize();
	bool Inside(double X[DIMENSION]);
};

#endif // !defined(CYLINDRICALCLUSTERINITIALIZER_H)
