#ifndef PARALLELEPIPEDCLUSTERINITIALIZER_H
#define PARALLELEPIPEDCLUSTERINITIALIZER_H

#include "DMFirstHeader.h"

#include "ClusterInitializer.h"
#include "math.h"

//! Creates a cluster in the form of a parallelepiped according to the vectors passed

/*!

  The cluster is centered using the SetCenter variable as usual. The corners will 
  be the center plus/minus the three boundary vectors. The format of the Boundary
  Vectors is as for SuperCellVectors in SkewPeriodicBoundaryConditions (allowing
  the latter to be directly used here) namely the coord index is first and the
  vector index second.


  \author nbailey
  \date August 2001

 */

class ParallelepipedClusterInitializer : public ClusterInitializer
{
public:
	ParallelepipedClusterInitializer(const double boundaryVectors[DIMENSION][DIMENSION],
                                     BravaisLatticeWithBasis *bravais);
	virtual ~ParallelepipedClusterInitializer();
	void SetBoundaryVectors(const double boundaryVectors[DIMENSION][DIMENSION]); 
    const double (&GetBoundaryVectors() const)[DIMENSION][DIMENSION] {return boundaryVectors;}
	
protected:
	double boundaryVectors[DIMENSION][DIMENSION];
	double inverseBoundaryVectors[DIMENSION][DIMENSION];
	double volume;
	int SetMaxSize();
	bool Inside(double X[DIMENSION]);
};

#endif // !defined(PARALLELEPIPEDCLUSTERINITIALIZER_H)
