#ifndef POLYCLUSTERINITIALIZER_H
#define POLYCLUSTERINITIALIZER_H

#include "DigitalMaterial.h"
#include "ClusterInitializer.h"
#include "DMArray.h"

//! PolyClusterInitializer initializes a poly cluster in two and three dimensions
/**
 * PolyClusterInitializer takes corners points and/or connectivity and intialize
 * a simple polygon in 2D or polyhedron in 3D. If the connectivity is not given, 
 * a convex region will be intialized based on the given corner points.
 *
 */

class PolyClusterInitializer : public ClusterInitializer  
{
public:
	PolyClusterInitializer(DMArray<double, DIMENSION>* corners, BravaisLatticeWithBasis *bravais, DMArray<int,DIMENSION>* connectivities = 0);
	virtual ~PolyClusterInitializer();
	const DMArray<double, DIMENSION>* GetCorners() const {return &corners;}
	void SetCorners(DMArray<double, DIMENSION>* corners); 
	const DMArray<int,DIMENSION>* GetConnectivities() const {return &connectivities;}
	void SetConnectivities(DMArray<int,DIMENSION>* connectivities); 
	//! Given query point X in the space, check whether the point is strictly inside 
	//! the polycluster
    bool StrictlyInside(double X[DIMENSION]) ;

protected:
#ifndef SWIG
	DMArray<double, DIMENSION> corners;
	DMArray<int, DIMENSION> connectivities;
	bool givenConnectivities;
	struct face
	{
		double point[DIMENSION], normal[DIMENSION];
	};
	std::vector<face* > faces;

    //! Return true iff point is not strict exterior
	bool Inside(double X[DIMENSION]);
	void SetConvexConnectivities();
	//! Returns the cos of the angle between x (supposed normalized) and a-b. 
	//! Used in the determination of the faces and in Inside.
	double XdotAminusB(double x[DIMENSION],double a[DIMENSION],double b[DIMENSION]); 
	int SetMaxSize();
	//! (TC) a temporary function returning the determinant of a matrix of dimension dim.
	double Determinant(double M[DIMENSION][DIMENSION], int dim); 
	int InsideRayCrossings() ;
    bool InsideConvex() ;
    double queryPoint[DIMENSION] ;
	// 0 Exterior, 1 Interior, 2 On the vertex, 3 On the edge, 4 On the face
	// XXX I should use enum for this later
	int InPolygon() ; 
    int InPolyhedron() ;
	bool InBox() ;
	char BoxTest( int n, double b[] ) ;
	double ComputeBoxRadius() ;
	double bmin[DIMENSION], bmax[DIMENSION] ;
	struct triBox
	{
		double minMax[2][DIMENSION];
	};
	std::vector<triBox* > Boxes ;
	void ComputeTriangleBoxes() ;
    char SegTri( int faceNum, double ray[]) ;
	char InTri3D( int Tri[], int m, double interSect[] ) ;
    char InTri2D( double Tp[][DIMENSION], double pp[] ) ;
    int AreaSign( double a[], double b[], double c[] ) ; 
    char SegPlane( int Tri[], double ray[], double interSect[], int *m) ;
    int	PlaneCoeff( int Tri[], double N[], double *D ) ;
    void NormalVec( double a[], double b[], double c[], double N[] ) ;
    double	Dot( double a[], double b[] ) ;
    void  SubVec( double a[], double b[], double c[] ) ;
    char SegTriCross( int Tri[], double ray[] ) ;
    int VolumeSign( double a[], double b[], double c[], double d[] ) ;
    void RandomRay( double ray[], double radius ) ;
#endif //SWIG
};

#endif // !defined(POLYCLUSTERINITIALIZER_H)
