// CompositeAtomsMover.h: interface for the CompositeAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#ifndef COMPOSITEATOMSMOVER_H
#define COMPOSITEATOMSMOVER_H


#include "DigitalMaterial.h"
#include "AtomsMover.h"


class CompositeAtomsMover : public AtomsMover  
{
public:
	CompositeAtomsMover(AtomsMover *mover1,AtomsMover *mover2,double tolerance);
	~CompositeAtomsMover();
	void Move(ListOfAtoms *atoms);
	
	void SetInterestingLeaves(int numberOfInterestingLeaves1, int numberOfInterestingLeaves2, int *interestingLeaves1, int *interestingLeaves2);
	
	double GetTolerance() const {return tolerance;}
	void SetTolerance(double t) {tolerance = t;}
	int GetMaxSteps() const {return maxSteps;}
	void SetMaxSteps(int n) {maxSteps = n;}
	
	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}
	
	int GetNIterations() const {return nIterations;}

	
	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);	
	const std::string &GetType() const
		{
			return className;
		}
	
private:
	// eventually could have an arbitrary numberof movers.
	AtomsMover *mover1;
	AtomsMover *mover2;

	//! The shape of the ghosts
	Shape ghostShape1;
	Shape ghostShape2;

	// which leaves on each branch are interesting to the other branch
	int numberOfInterestingLeaves1;
	int numberOfInterestingLeaves2;
	int *interestingLeaves1;
	int *interestingLeaves2;
	

	int nParameters;
	double tolerance;
	int maxSteps;
	int nAtoms;
	int nIterations;
	bool verbose;
	static const std::string className;
};

#endif //  COMPOSITEATOMSMOVER_H
