// ConstraintVerletAtomsMover.h: interface for the ConstraintVerletAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CONSTRAINTVERLETATOMSMOVER_H
#define CONSTRAINTVERLETATOMSMOVER_H


#include "DigitalMaterial.h"
#include "AtomsMover.h"
#include "DMArray.h"
#include "Constraint.h"

class Potential;

class ExternalStress;


class ConstraintVerletAtomsMover : public AtomsMover
{
 public:
	ConstraintVerletAtomsMover(Potential *potential = 0,double dt = 1., double tolerance = 1.e-6);
	~ConstraintVerletAtomsMover();

	double GetTolerance() const {return tolerance;}
	void SetTolerance(double t) {tolerance = t;}
	int GetMaxSteps() const {return maxSteps;}
	void SetMaxSteps(int n) {maxSteps = n;}
	double GetDt() const {return dt;}
	void SetDt(double t) {dt = t;}

	void Move(ListOfAtoms *atoms);
   
	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}

	void SetMinimize() {minimize = true;}
	void UnSetMinimize() {minimize = false;}
	
	// Accessing "return" information
	double GetFinalTFS() const {return generalizedForceSq;}
	int GetNIterations() const {return nIterations;}
    double GetResidualForceSq() const {return residualForceSq;}


	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
		{
			return className;
		}
    
 protected:
	void CalculateGeneralizedForces(ListOfAtoms *atoms, Constraint *constraint,DMArray<double,DIMENSION> *forces);
    void IncrementGeneralizedVelocitiesFromForces(ListOfAtoms *atoms);
	void Allocate(int nDOFs);
	
	//! if verbose is set, print out status every \a STEPSBETWEENDISPLAY iterations
	static const int STEPSBETWEENDISPLAY;

	bool minimize;
	bool verbose;
	int nIterations;
	double dt;
	int nDOFs;
	double tolerance;
	int maxSteps;

    Potential *potential;

    double residualForceSq;
	double generalizedForceSq;

	double *generalizedCoordinates;
	double *generalizedVelocities;
	double *generalizedForces;

    double *kineticMatrix;
private:
	static const std::string className;
};



#endif // CONSTRAINTVERLETATOMSMOVER_H
