// ExtendedQuickMinAtomsMover.h: interface for the ExtendedQuickMinAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#ifndef EXTENDEDQUICKMINATOMSMOVER_H
#define EXTENDEDQUICKMINATOMSMOVER_H

#include "DigitalMaterial.h"
#include "AtomsMover.h"
class Potential;

#include "AtomsExtension.h"
class QuickMinAtomsMover;

class ExtendedQuickMinAtomsMover : public AtomsMover  
{
public:
	ExtendedQuickMinAtomsMover(AtomsExtension *extension = 0,QuickMinAtomsMover *qm = 0,Potential *potential = 0,double dt = 1.,double tolerance = 1.e-6,int nBdyIterations = 1);
	~ExtendedQuickMinAtomsMover();
	void Move(ListOfAtoms *atoms);
	
	void SetInitialParameters(double *initialParameters);
	const double *GetFinalParameters() const {return (const double*)parameters;}
	
	double GetTolerance() const {return tolerance;}
	void SetTolerance(double t) {tolerance = t;}
	int GetMaxIterations() const {return maxIterations;}
	void SetMaxIterations(int n) {maxIterations = n;}
	int GetBoundaryIterations() const {return nBdyIterations;}
	void SetBoundaryIterations(int nBdyIterations) {this->nBdyIterations = nBdyIterations;}
	double GetDt() const {return dt;}
	void SetDt(double t) {dt = t;}
	

	const double *GetKineticMatrix(ListOfAtoms *atoms, const double *parameters);

	void SetAtomsExtension(AtomsExtension *atomsExtension) {this->atomsExtension = atomsExtension; nParameters = atomsExtension->GetNParameters();}
  AtomsExtension *GetAtomsExtension() {return atomsExtension;}
	
	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}
	
	// Accessing "return" information
	double GetConvergence() const {return convergence;}
	//double GetFinalTFS() const {return finalTFS;}
	int GetNIterations() const {return nIterations;}
	
	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
		{
			return className;
		}
	// The following function is for debugging purposes
	//void CheckEnergyForce(ListOfAtoms *atoms);

private:
	// Private member functions
	void Allocate(const DMArray<double, DIMENSION> *array);
	void EstimateModeFrequencies(ListOfAtoms *atoms);

	// Pointers to external objects
	QuickMinAtomsMover *qm;
	AtomsExtension *atomsExtension;
	Potential *potential;

	// Internal arrays (size depends on the LOA and the AtomsExtension
	DMArray<double,DIMENSION> *forces;
	double *parameters;
	double *parameterVelocities;
	double *parameterForces;
    double *kineticMatrix;
	// variables related to the sizes of the arrays
	int nParameters;
	int nAtoms;	

	// user-set parameters
	double dt;
	double convergence;
	double tolerance;
	int nIterations;
	int maxIterations;
	int nBdyIterations;
	bool verbose;
	static const std::string className;
};

#endif //  EXTENDEDQUICKMINATOMSMOVER_H
