// HooverAtomsMover.h: interface for the HooverAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

// From "Time-reversible equilibrium and nonequilibrium isothermal-isobaric simulations
// with centered-difference Stoermer algorithms
// B.L. Holian, A.J. DeGroot, W.G. Hoover, and C.G. Hoover,
// Phys Rev A 41, 4552 - 4553 (1990)

#ifndef HOOVERATOMSMOVER_H
#define HOOVERATOMSMOVER_H

#include "DigitalMaterial.h"
#include "AtomsMover.h"
#include "Potential.h"

class HooverAtomsMover : public AtomsMover  
{
public:
	HooverAtomsMover(Potential *potential = 0, double temperature = 0.0, double dt=0.01, double tau = 1.0, int nSteps=1);
	virtual ~HooverAtomsMover() {}

	void Move(ListOfAtoms *atoms);

	// Time step for Verlet algorithm
	double GetDt() const {return dt;}
	void SetDt(double dt) {this->dt = dt;}

	// Target temperature
	double GetTemperature() const {return temperature;}
	void SetTemperature(double temperature) {this->temperature = temperature;}

	// Time-reversible friction coefficient
	double GetZeta() const {return zeta;}
	void SetZeta(double zeta) {this->zeta = zeta;}

	// Thermostat relaxation time
	double GetTau() const {return tau;}
	void SetTau(double tau) {this->tau = tau;}

	// Number of time-steps per call.
	int GetNSteps() const {return nSteps;}
	void SetNSteps(int nSteps) {this->nSteps = nSteps;}
	
	const std::string &GetType() const
		{
			return className;
		}
#ifndef SWIG
    void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	
#endif //SWIG
protected:
	void Allocate(int nAtoms, const DMArray<double, DIMENSION> *array);
	double dt;
	int nAtoms;		// Tells how large a force array is stored
	int nSteps;
	DMArray<double, DIMENSION> *forces;

	double zeta;	// Time-reversible friction coefficient
	double tau;		// Thermostat relaxation time
	double temperature; // Target temperature
	static const std::string className;
};

#endif // !defined(HOOVERATOMSMOVER_H)
