// MinimizerAtomsMover.h: interface for the MinimizerAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MINIMIZERATOMSMOVER_H
#define MINIMIZERATOMSMOVER_H

#include <math.h>
#include "DigitalMaterial.h"
#include "AtomsMover.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "Minimizer.h"
#include "Minimizable.h"
class BoundaryConditions;
class NeighborLocator;
class ExternalWork;

//! Performs ConjugateGradient minimization on a ListOfAtoms.
/*! Needs to be passed a potential when constructed.
	*/

class MinimizerAtomsMover : public AtomsMover
{
public:
	void Move(ListOfAtoms *atoms);
	MinimizerAtomsMover(Minimizer *minimizer = 0, Potential *potential = 0,ExternalWork *work = 0);
	virtual ~MinimizerAtomsMover();
	Minimizer *GetMinimizer() {return minimizer;}

	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}
	

#ifndef SWIG
	class AtomsMinimizable : public Minimizable
	{
	public:
		AtomsMinimizable(ListOfAtoms *atoms,Potential *potential,ExternalWork *work, bool verbose) 
			{
				this->atoms = atoms;
				// ajd27 forces are a sibling to the positions.
				forces = new DMArray<double, DIMENSION>(
					(atoms->GetCartesianPositions()));
				this->potential = potential;
				this->work = work;
				bc = atoms->GetBoundaryConditions();
				neighborLocator = atoms->GetNeighborLocator();
				gradients = new double[atoms->GetNumber()*DIMENSION];
				this->verbose = verbose;
			}
		~AtomsMinimizable()
			{
				delete gradients;
				delete forces; 
			}
		double ObjectiveFunction(double *parameters);
		double *Gradient(double *parameters);
		int GetNParameters() {return atoms->GetNumber()*DIMENSION;}
	private:
		ListOfAtoms *atoms;
		DMArray<double, DIMENSION> *forces;
		double *gradients;
		Potential *potential;
		const BoundaryConditions *bc;
		NeighborLocator *neighborLocator;
		ExternalWork *work;
		bool verbose;
	};
	Minimizable *atomsMinimizable;


	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
		{
			return className;
		}
#endif // SWIG

private:
	Minimizer 	*minimizer;
	Potential *potential;
	int nIterations;
	ExternalWork *work;
	static const std::string className;
	bool verbose;
};

#endif // !defined(MINIMIZERATOMSMOVER_H)
