// QuickMinAllCoordinates.h: interface for the QuickMinAllCoordinates class.
//
//////////////////////////////////////////////////////////////////////

#ifndef QUICKMINALLCOORDINATES_H
#define QUICKMINALLCOORDINATES_H

#include "DigitalMaterial.h"
#include "QuickMinAtomsMover.h"
#include "Potential.h"

//! Version of QuickMin which acts on the full DN-dimensional velocity-vector

/*!
  This is another derived class of QuickMinAtomsMover, which rather than making
  a decision for each atom, acts on the full DN-dimensional velocity vector.
  If it only zeroed the velocity when the total dot-product of force and 
  velocity was negative, the efficiency would be low for larger systems
  because this condition would be rare. So it also performs a projection of
  the velocity vector in the direction of the force vector.

*/

class QuickMinAllCoordinates : public QuickMinAtomsMover 
{
public:
	QuickMinAllCoordinates(Potential *potential = 0,double dt = 1.,double tolerance = 1.e-6) : QuickMinAtomsMover(potential, dt, tolerance) {}
protected:
	void AdjustVelocities(const DMArray<double, DIMENSION> *forces, const DMArray<double, DIMENSION> *velocities);

private:
	static const std::string className;
};

#endif //  QUICKMINALLCOORDINATES_H
