// VerletAtomsMover.cpp: implementation of the VerletAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "VerletAtomsMover.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "SerialBuilder.h"


#include "DMLastHeader.h"

static SerialBuilder<VerletAtomsMover> *_Verletcreator = 
    new SerialBuilder<VerletAtomsMover>("VerletAtomsMover");

const std::string VerletAtomsMover::className = std::string("VerletAtomsMover");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

VerletAtomsMover::VerletAtomsMover(Potential *potential, double dt, int nSteps)
{
	this->potential = potential;
	this->dt = dt;
	this->nSteps = nSteps;

	forces = 0;
	SetTime();
}

VerletAtomsMover::~VerletAtomsMover()
{
	if (forces) delete forces;
}

void VerletAtomsMover::Move(ListOfAtoms *atoms)	// Verlet
{
	double halfDt = 0.5*dt;
	if (!(forces && (forces->GetShape() == atoms->GetShape())))
		Allocate(atoms->GetCartesianPositions());
	atoms->IncrementCartesianPositions(atoms->GetCartesianVelocities(), halfDt);
	
	time += halfDt;

	for (int i=0; i<nSteps-1; i++)
	{
		potential->CalculateForces(atoms, forces);
		atoms->AdjustForces(forces);
		atoms->IncrementCartesianMomenta(forces,dt);
		atoms->AdjustVelocities();
		time += halfDt;
		atoms->IncrementCartesianPositions(atoms->GetCartesianVelocities(),dt);
		time += halfDt;
	}
	potential->CalculateForces(atoms, forces);
	atoms->AdjustForces(forces);
	atoms->IncrementCartesianMomenta(forces,dt);
	atoms->AdjustVelocities();

	time += halfDt;
	atoms->IncrementCartesianPositions(atoms->GetCartesianVelocities(), halfDt);
}

void VerletAtomsMover::Allocate(const DMArray<double, DIMENSION> *array)
{
	if (forces) delete forces;
	forces = new DMArray<double, DIMENSION>(array);
}



////////////////////////////////////////////////////////////////////////////////
/// METHODS RELATED TO SERIALIZATION
////////////////////////////////////////////////////////////////////////////////


void VerletAtomsMover::Save(DMWriter *writer) const
{
	AtomsMover::Save(writer);	

	writer->PutDouble(time, "time");
	writer->PutDouble(dt, "dt");
	writer->PutInt(nSteps, "nSteps");
}

void VerletAtomsMover::Load(DMReader *reader)
{
	AtomsMover::Load(reader);

	SetTime(reader->GetDouble("time"));
	SetDt(reader->GetDouble("dt"));
	SetNSteps(reader->GetInt("nSteps"));
}
