// VerletThermalizeAtomsMover.h: interface for the VerletThermalizeAtomsMover class.
//
//////////////////////////////////////////////////////////////////////

// Randomize kinetic energies to current temperature, then run nSteps of the
// Verlet algorithm

#ifndef VERLETTHERMALIZEATOMSMOVER_H
#define VERLETTHERMALIZEATOMSMOVER_H

#include "AtomsMover.h"
#include "Potential.h"
#include "Rand.h"

class VerletThermalizeAtomsMover : public AtomsMover  
{
public:
	VerletThermalizeAtomsMover(Potential *potential = 0, double temperature = 0.0, double dt = 0.01, int nSteps = 1);
	virtual ~VerletThermalizeAtomsMover() {}
	void Move(ListOfAtoms *atoms);

	// Time step for Verlet algorithm
	double GetDt() const {return dt;}
	void SetDt(double dt) {this->dt = dt;}

	// Target temperature
	double GetTemperature() const {return temperature;}
	void SetTemperature(double temperature) {this->temperature = temperature;}

	// Number of time-steps per call.
	int GetNSteps() const {return nSteps;}
	void SetNSteps(int nSteps) {this->nSteps = nSteps;}
	
	// Re-initialize random number generator
	void SetRandomSeed(int seed) 
	{
		rand.SetSeed(seed);
	}

	int GetRandomSeed() const
	{
		return rand.GetOriginalSeed();
	}

#ifndef SWIG
    void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
    {
        return className;
    }
#endif // SWIG

protected:
	void Allocate(int nAtoms, const DMArray<double, DIMENSION> *array);
	double dt;
	int nAtoms;		// Tells how large a force array is stored
	int nSteps;
	DMArray<double, DIMENSION> *forces;
	DMArray<double, DIMENSION> *newVelocities;

	double temperature; // Target temperature
	Rand rand;			// Random number generator
	static const std::string className;
};

#endif // !defined(VERLETTHERMALIZEATOMSMOVER_H)
