%{
#include "FreeBoundaryConditions.h"
#include "SimplePeriodicBoundaryConditions.h"
#include "SmartPeriodicBoundaryConditions.h"
#include "SkewPeriodicBoundaryConditions.h"
%}

%nodefault BoundaryConditions;
class BoundaryConditions : public Serializable {};
%include "FreeBoundaryConditions.h"

%nodefault PeriodicBoundaryConditions;
class PeriodicBoundaryConditions:public BoundaryConditions
{
public:
   double GetLength(int coord);
};
%include "SimplePeriodicBoundaryConditions.h"


class SkewPeriodicBoundaryConditions : public BoundaryConditions
{

public:
    void SetSuperCellVectors(const double superCellVectors[DIMENSION][DIMENSION]);
    bool GetWrap(int coord) const;
%extend
{
    SkewPeriodicBoundaryConditions(PyObject *wrap=0)
    {
        if(wrap == 0) return new SkewPeriodicBoundaryConditions();
        bool wwrap[DIMENSION];
        if(PyList_Check(wrap) && PyList_Size(wrap) == DIMENSION)
        {
            for(int i=0;i<DIMENSION;++i)
            {
                PyObject *o = PyList_GetItem(wrap,i);
                if(PyInt_Check(o)) wwrap[i] = (PyInt_AsLong(o) != 0);
                else
                {
                    PyErr_SetString(PyExc_TypeError,"list must contain int");
                    return NULL;
                }
            }
        }
	else
	{
                PyErr_SetString(PyExc_TypeError,"not a list of the right size");
                return NULL;
        }
        return new SkewPeriodicBoundaryConditions(wwrap);
     }
     PyObject *GetSuperCellVector(int vec)
     {
	PyObject *vector = PyList_New(DIMENSION);
	for( int coord = 0; coord < DIMENSION; ++coord )
	PyList_SetItem(vector, coord, PyFloat_FromDouble(self->GetSuperCellVectors()[coord][vec]));

	return vector;
     }
}



};

class SmartPeriodicBoundaryConditions: public PeriodicBoundaryConditions
{
public:
    void SetBoxCenter(const double center[DIMENSION]);
    void SetLength(const double length[DIMENSION]);
    void SetLowerEdge(const double lowerEdges[DIMENSION]);
    void SetUpperEdge(const double upperEdges[DIMENSION]);
    bool GetWrap(int coord) const;
%extend
{
    SmartPeriodicBoundaryConditions(PyObject *wrap=0)
    {
        if(wrap == 0) return new SmartPeriodicBoundaryConditions();
        bool wwrap[DIMENSION];
        if(PyList_Check(wrap) && PyList_Size(wrap) == DIMENSION)
        {
            for(int i=0;i<DIMENSION;++i)
            {
                PyObject *o = PyList_GetItem(wrap,i);
                if(PyInt_Check(o)) wwrap[i] = (PyInt_AsLong(o) != 0);
                else
                {
                    PyErr_SetString(PyExc_TypeError,"list must contain int");
                    return NULL;
                }
            }
        }
	else
	{
                PyErr_SetString(PyExc_TypeError,"not a list of the right size");
                return NULL;
        }
        return new SmartPeriodicBoundaryConditions(wwrap);
     }
}
};
