// SimplePeriodicBoundaryConditions.cpp: implementation of the SimplePeriodicBoundaryConditions class.
//
//////////////////////////////////////////////////////////////////////
#include <math.h>

#include "DigitalMaterial.h"

#include "SimplePeriodicBoundaryConditions.h"
#include "DMArray.h"
#include "SerialBuilder.h"

#include "DMLastHeader.h"

static SerialBuilder<SimplePeriodicBoundaryConditions> *_SimplePBCcreator = 
	new SerialBuilder<SimplePeriodicBoundaryConditions>("SimplePeriodicBoundaryConditions");

const std::string SimplePeriodicBoundaryConditions::className = std::string("SimplePeriodicBoundaryConditions");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SimplePeriodicBoundaryConditions::SimplePeriodicBoundaryConditions()
{
	for(int coord=0;coord<DIMENSION;coord++)
	{
		length[coord] = 1.0; // default; change using SetLength
	}
}
SimplePeriodicBoundaryConditions::SimplePeriodicBoundaryConditions(const double l[DIMENSION])
{
	SetLength(l);
}

SimplePeriodicBoundaryConditions::~SimplePeriodicBoundaryConditions()
{

}


void SimplePeriodicBoundaryConditions::EnforceBoundaryConditions(DMArray<double, DIMENSION> *positions) const
{
	DMArrayIterator<double, DIMENSION> posIter(positions);
	for(posIter.Init(); !posIter.IsDone(); posIter.Next())
	{
		double *atomPos = posIter.Current();
		for(int coord=0;coord<DIMENSION;coord++)
		{
			if(atomPos[coord] >= length[coord])
			{
				atomPos[coord] -= length[coord];			// Do this if fmod is more expensive than two "if"s.
				if (atomPos[coord] >= length[coord])
				{
					atomPos[coord] = fmod(atomPos[coord], length[coord]);
				}
			}
			if(atomPos[coord] < 0.0)
			{
				atomPos[coord] += length[coord];			// Do this if fmod is more expensive than two "if"s.
				if(atomPos[coord] < 0.0)
				{
					atomPos[coord] = fmod(atomPos[coord],length[coord])+length[coord];
				}
			}
		}
	}

/*  this is the previous version
	for(int i=0;i<nAtoms;i++)
	{
		for(int coord=0;coord<DIMENSION;coord++)
		{
			if(positions[i][coord] >= length[coord])
			{
				positions[i][coord] -= length[coord];			// Do this if fmod is more expensive than two "if"s.
				if (positions[i][coord] >= length[coord])
				{
					positions[i][coord] = fmod(positions[i][coord], length[coord]);
				}
			}
			if(positions[i][coord] < 0.0)
			{
				positions[i][coord] += length[coord];			// Do this if fmod is more expensive than two "if"s.
				if(positions[i][coord] < 0.0)
				{
					positions[i][coord] = fmod(positions[i][coord],length[coord])+length[coord];
				}
			}
		}
	}
*/	
}


void SimplePeriodicBoundaryConditions::EnforceBoundaryConditions(double *position) const
{
  
  for(int coord=0;coord<DIMENSION;coord++)
    {
      if(position[coord] >= length[coord])
	{
	  position[coord] -= length[coord];			// Do this if fmod is more expensive than two "if"s.
	  if (position[coord] >= length[coord])
	    {
	      position[coord] = fmod(position[coord], length[coord]);
	    }
	}
      if(position[coord] < 0.0)
	{
	  position[coord] += length[coord];			// Do this if fmod is more expensive than two "if"s.
	  if(position[coord] < 0.0)
	    {
	      position[coord] = fmod(position[coord],length[coord])+length[coord];
	    }
	}
    }
  
  
}

void SimplePeriodicBoundaryConditions::DifferenceBoundaryConditions(DMArray<double, DIMENSION> *displacements) const
{
	DMArrayIterator<double, DIMENSION> dispIter(displacements);	
	for(dispIter.Init(); !dispIter.IsDone(); dispIter.Next())
	{
		double *tmp = dispIter.Current();
		for(int coord=0;coord<DIMENSION;coord++)
		{
			if(tmp[coord] >= 0.5*length[coord]) tmp[coord] -= length[coord];
			if(tmp[coord] < -0.5*length[coord]) tmp[coord] += length[coord]; 
		}
	}
}


void SimplePeriodicBoundaryConditions::DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const
{
   for(int i=0;i<nAtoms;i++)
     for(int coord=0;coord<DIMENSION;coord++)
     {
     if(displacements[i][coord] >= 0.5*length[coord])
	     displacements[i][coord] -= length[coord];
     if(displacements[i][coord] < -0.5*length[coord])
	     displacements[i][coord] += length[coord]; 
     }

}

BoundaryConditions *SimplePeriodicBoundaryConditions::Copy() const
{
  return new SimplePeriodicBoundaryConditions(length);

}


