// SmartPeriodicBoundaryConditions.h: interface for the SmartPeriodicBoundaryConditions class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SMARTPERIODICBOUNDARYCONDITIONS_H
#define SMARTPERIODICBOUNDARYCONDITIONS_H

#include "DigitalMaterial.h"

#include "PeriodicBoundaryConditions.h"
#include "DMArray.h"
/**
 * This is a "smarter" periodic boundary conditions class where you can choose not to have 
 * the periodicity along some directions. This is specified with the "wrap" variable in the 
 * constructor. We can also define the center of the box, of set its corners.
 * \date 11 May 2000
 * \author Thierry Cretegny
 * \ingroup moleculardynamics
**/
class SmartPeriodicBoundaryConditions : public PeriodicBoundaryConditions  
{
public:
	SmartPeriodicBoundaryConditions(const bool wrap[DIMENSION] = 0);

	virtual ~SmartPeriodicBoundaryConditions();

	void SetBoxCenter(const double center[DIMENSION]);
	void SetLength(const double length[DIMENSION]);
	void SetLowerEdge(const double lowerEdges[DIMENSION]);
	void SetUpperEdge(const double upperEdges[DIMENSION]);
	bool GetWrap(int coord) const {return wrap[coord];}

	virtual void EnforceBoundaryConditions(DMArray<double ,DIMENSION> *positions) const;
	virtual void EnforceBoundaryConditions(double *position) const;
	void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const;
	void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *displacements) const;

	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}
	BoundaryConditions *Copy() const;
private:
	bool wrap[DIMENSION];

	double lowerEdge[DIMENSION];
	double upperEdge[DIMENSION];
	double center[DIMENSION];
	static const std::string className;
};

#endif // !defined(SMARTPERIODICBOUNDARYCONDITIONS_H
