#include "DMFirstHeader.h"

#include "DigitalMaterial.h"

#include "BravaisLatticeWithBasis.h"
#if DIMENSION == 3


class CubicLattice: public BravaisLatticeWithBasis
{
public:
    void SetLatticeConstant(double a);
    double GetLatticeConstant() const {return latticeConstant;}
    CubicLattice(double a);
    ~CubicLattice(){};
    const std::string &GetType() const {return className;}
protected:
    double latticeConstant;
private:
	static const std::string className;
};


class FCCLattice:public CubicLattice
{
public:
    FCCLattice(double a=1.0);
    ~FCCLattice(){}
    void AddAtomInCell(TinyVector<double, DIMENSION> atomCoord,
                               std::string name="");
    const std::string &GetType() const {return className;}
private:
	static const std::string className;
};

class PrimitiveFCCLattice:public BravaisLatticeWithBasis
{
public:
    PrimitiveFCCLattice(double a=1.0);
    ~PrimitiveFCCLattice(){};
    //! restores the usual definition of the lattice vectors.
    void ResetOrientation(); 
    const std::string &GetType() const
	{
		return className;
	}
    void SetLatticeConstant(double a);
    double GetLatticeConstant() const {return latticeConstant;}
protected:
    //! The lattice constant has the same meaning as in FCCLattice
    double latticeConstant;
private:
	static const std::string className;
};

class BCCLattice:public CubicLattice
{
public:
    BCCLattice(double a=1.0);
    void AddAtomInCell(TinyVector<double, DIMENSION> atomCoord,
                       std::string name="");
    const std::string &GetType() const
	{
		return className;
	}
private:
	static const std::string className;
};

class DiamondLattice: public FCCLattice
{
public:
    //! Note that the lattice spacing is the one of the uderlying cubic lattice
    DiamondLattice(double a=1.0);
	~DiamondLattice(){}
    void AddAtomInCell(TinyVector<double, DIMENSION> atomCoord,
                       std::string name="");
    const std::string &GetType() const {return className;}
private:
	static const std::string className;
};

class TriclinicLattice:public GeneralLattice
{
public:
    TriclinicLattice(const double latticeConstants[DIMENSION],
                     const double anglesInUnitCell[DIMENSION]);
    //! Default constructor needed for serialization
    TriclinicLattice();
    ~TriclinicLattice(){};

    void SetLatticeConstants(const double latticeConstants[DIMENSION]);
    void SetAnglesInUnitCell(const double anglesInUnitCell[DIMENSION]);

    const double (&GetLatticeConstants()const) [DIMENSION] {return latticeConstants;}
    const double (&GetAnglesInUnitCell()const) [DIMENSION] {return anglesInUnitCell;}

    //! Load the class.
    void Load(DMReader *reader) ;

    //! Get the string name of the class for saving or loading.
    const std::string &GetType() const {return className;}
protected:

    double latticeConstants[DIMENSION];
    double anglesInUnitCell[DIMENSION];
    void Set3DLatticeVectors();
private:
	static const std::string className;
};


#endif // DIMENSION == 3
