// Constraint.cpp: implementation of the Constraints class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "Constraint.h"
#include "ListOfAtoms.h"
#include "DMArray.h"

#include "DMLastHeader.h"

const std::string Constraint::className = std::string("Constraint");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Constraint::Constraint()
{
	atoms = 0;
}

Constraint::~Constraint()
{

}

Constraint *Constraint::Copy() const
{
	return new Constraint();
}

int Constraint::NumberGeneralizedCoordinates()
{
	return DIMENSION*atoms->GetNumber();
}


void Constraint::IncrementGeneralizedCoordinate(int n,double inc)
{
	atoms->IncrementCartesianPosition(n/DIMENSION,n%DIMENSION,inc);
}


void Constraint::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{
	AdjustForces(forces); // has an effect in derived classes which overload
	// AdjustForces but don't otherwise need to overload CalculateGeneralizedForces


	// this is the same code as is in the DynamicListOfAtoms function, which gets executed if there are no constraints. The code shouldn't be duplicated!
	int index = 0;
	DMArrayIterator<double, DIMENSION> forcesIter(forces);
	for(forcesIter.Init();!forcesIter.IsDone();forcesIter.Next())
		for(int coord=0;coord<DIMENSION;++coord)
		{
			generalizedForces[index] = forcesIter.Current()[coord];
			++index;
		}
}

void Constraint::GetGeneralizedCoordinates(double *generalizedCoordinates)
{
	DMArrayIterator<double,DIMENSION> posIter(atoms->GetCartesianPositions());
	int index = 0;
	for(posIter.Init();!posIter.IsDone();posIter.Next())
		for(int coord=0;coord<DIMENSION;++coord)
		{			
			generalizedCoordinates[index++] = posIter.Current()[coord];
		}
}



void Constraint::SetGeneralizedCoordinates(double *generalizedCoordinates)
{
	DMArray<double,DIMENSION> newPositions(atoms->GetCartesianPositions()->GetShape());
	DMArrayIterator<double,DIMENSION> posIter(&newPositions);
	int index = 0;
	for(posIter.Init();!posIter.IsDone();posIter.Next())
		for(int coord=0;coord<DIMENSION;++coord)
		{			
			posIter.Current()[coord] = generalizedCoordinates[index++];
		}
	
	atoms->SetCartesianPositions(&newPositions);
}


void Constraint::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{
	for(int atomIdx=0, index=0;atomIdx<atoms->GetNumber();++atomIdx)
		for(int coord=0;coord<DIMENSION;++coord)
		{
			// it's diagonal: zero except for coord == coord2
			for(int coord2=0;coord2<DIMENSION;++coord2)
				gradient[index][coord2] = 0.;
			gradient[index++][coord] = 1.;// only increment index here
		}
}

