// ExternalForceConstraint.h: interface for the ExternalForceConstraint class.
//
//////////////////////////////////////////////////////////////////////

#ifndef EXTERNALFORCECONSTRAINT_H
#define EXTERNALFORCECONSTRAINT_H

#include "DigitalMaterial.h"
#include "Constraint.h"

//! Allows external forces to be added to a ListOfAtoms

/*!

  This constraint allows you to add an external force (when you call 
  AdjustForces). So it can be used on a ListOfAtoms forming a boundary, say,
  where external force or stress is to be applied.

  Degrees of freedom are otherwise unconstrained so no need to overload the
  GeneralizedCoordinate functions.

  Previously (before the date below) existed as the class "Handle" in 
  Constraint.h

  
  \ingroup MolecularDynamics
  \author Nicholas Bailey
  \date March 1, 2001
  
 */ 



class ExternalForceConstraint : public Constraint
{
public:
	ExternalForceConstraint(); 
	virtual ~ExternalForceConstraint() ;
	Constraint *Copy() const;
	void SetExternalForce(const double extForce[DIMENSION])
		{for(int coord=0;coord<DIMENSION;++coord) externalForce[coord]=extForce[coord];}
	const double *GetExternalForce() const {return externalForce;}
	void AdjustForces(DMArray<double, DIMENSION> *forces);


	const std::string &GetType() const
	{
		return className;
	}

protected:
	double externalForce[DIMENSION];
	static const std::string className;
};


#endif // !defined(EXTERNALFORCECONSTRAINT_H)
