// FixedCenterOfMass.cpp: implementation of the FixedCenterOfMass class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "FixedCenterOfMass.h"
#include "ListOfAtoms.h"
#include "DMArray.h"

#include "DMLastHeader.h"


const std::string FixedCenterOfMass::className = std::string("FixedCenterOfMass");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


FixedCenterOfMass::FixedCenterOfMass(const double center[DIMENSION])
{
    this->SetCenter(center);
}

void FixedCenterOfMass::SetCenter(const double center[DIMENSION])
{
    for(int coord=0 ; coord<DIMENSION; ++ coord) 
        this->center[coord] = center[coord];
}

void FixedCenterOfMass::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
    double avg[DIMENSION];
    positions->Average(avg);
    for(int coord=0; coord<DIMENSION; ++coord)
        avg[coord] = center[coord] - avg[coord];
    positions->Increment(avg);
}


int FixedCenterOfMass::NumberGeneralizedCoordinates()
{
    return DIMENSION*(atoms->GetNumber() - 1);
}

void FixedCenterOfMass::SetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("FixedCenterOfMass","SetGeneralizedCoordinates","Not Implemented Yet!");
}


void FixedCenterOfMass::GetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("FixedCenterOfMass","GetGeneralizedCoordinates","Not Implemented Yet!");
    
}


void FixedCenterOfMass::IncrementGeneralizedCoordinate(int n,double inc)
{
    throw DMException("FixedCenterOfMass","IncrementGeneralizedCoordinate",
        "Not Implemented Yet!");
}


void FixedCenterOfMass::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces,
                                                   double *generalizedForces)
{
    throw DMException("FixedCenterOfMass","CalculateGeneralizedForces","Not Implemented Yet!");
}

void FixedCenterOfMass::DerivativesWRTGeneralizedCoordinates(int atom,
                                                             double (*gradient)[DIMENSION])
{
    throw DMException("FixedCenterOfMass","DerivativesWRTGeneralizedCoordinates",
        "Not Implemented Yet!");
}
