// FixedCenterOfMass.h: interface for the FixedCenterOfMass class.
//
//////////////////////////////////////////////////////////////////////

#ifndef FIXEDCENTEROFMASS_H
#define FIXEDCENTEROFMASS_H

#include "DigitalMaterial.h"
#include "Constraint.h"

//! fixed COM: number of DOFS is DIMENSION*(nAtoms-1)

/*!
  
  \author Thierry Cretegny

 */

class FixedCenterOfMass: public Constraint
{
public:
	FixedCenterOfMass(const double center[DIMENSION]);
	void AdjustPositions(DMArray<double, DIMENSION> *positions);
	void SetCenter(const double center[DIMENSION]);

	// GeneralizedCoordinate functions: most throw errors right now (March 01)
	int NumberGeneralizedCoordinates();
	void SetGeneralizedCoordinates(double *generalizedCoordinates) ;
    void GetGeneralizedCoordinates(double *generalizedCoordinates); 
	void IncrementGeneralizedCoordinate(int n,double inc);
    void CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces);
    void DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION]);


	const std::string &GetType() const
	{
		return className;
	}

private:
	double center[DIMENSION];
	static const std::string className;
};

#endif // !defined(FIXEDCENTEROFMASS_H)
