// PartialFixedBody.h: interface for the PartialFixedBody class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PARTIALFIXEDBODY_H
#define PARTIALFIXEDBODY_H

#include "DigitalMaterial.h"
#include "Constraint.h"
#include "FixedBody.h"

class PartialFixedBody : public Constraint
{
public:
	PartialFixedBody(const DMArray<double, DIMENSION> *positions, const int directions[DIMENSION]);
        ~PartialFixedBody();
	Constraint *Copy() const;

	void SetFixedPositions(const DMArray<double, DIMENSION> *positions);
	void SetFixedDirections(const int directions[DIMENSION]);
	void AdjustPositions(DMArray<double, DIMENSION> *positions);
	void AdjustForces(DMArray<double, DIMENSION> *forces);
	double GetTotalForce(int coord);

	int NumberGeneralizedCoordinates();
	// these all do nothing, since there are no generalized coordinates
	// we don't want them to throw errors because they will be involved
	// in loops over branches, in which case they will be passed over
	// and the arrays wfilled in/used by the next leaf/branch which has
	// actual degrees of freedom
	void SetGeneralizedCoordinates(double *generalizedCoordinates) ;
    void GetGeneralizedCoordinates(double *generalizedCoordinates); 
	void IncrementGeneralizedCoordinate(int n,double inc);
#ifndef SWIG
    void CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces);
    void DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION]);
#endif // SWIG
	const std::string &GetType() const
	{
		return className; 
	}


protected:
	DMArray<double, DIMENSION> *fixedPositions;
	int fixedDirections[DIMENSION];
	static const std::string className;
	double totalForce[DIMENSION];
};



#endif // !defined(PARTIALFIXEDBODY_H)
