// PartialFixedCOM.cpp: implementation of the PartialFixedCOM class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "PartialFixedCOM.h"
#include "ListOfAtoms.h"
#include "DMArray.h"

#include "DMLastHeader.h"


const std::string PartialFixedCOM::className = std::string("PartialFixedCOM");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


PartialFixedCOM::PartialFixedCOM(bool fixedDir[DIMENSION], const double center[DIMENSION])
{
	nFixed = 0;
	for(int coord=0 ; coord<DIMENSION; ++ coord) 
	{
		this->fixedDir[coord] = fixedDir[coord];
		if (fixedDir[coord])
			++nFixed;
	}
    this->SetCenter(center);
}


PartialFixedCOM::PartialFixedCOM(bool fixedDir[DIMENSION], const DMArray<double, DIMENSION> *initialPositions)
{
	double avg[DIMENSION];
	initialPositions->Average(avg);
	nFixed = 0;
	for(int coord=0 ; coord<DIMENSION; ++ coord) 
	{
		this->fixedDir[coord] = fixedDir[coord];
		if (fixedDir[coord])
			++nFixed;
		this->center[coord] = avg[coord];
	}
}

void PartialFixedCOM::SetCenter(const double center[DIMENSION])
{
    for(int coord=0 ; coord<DIMENSION; ++ coord) 
        this->center[coord] = center[coord];
}

void PartialFixedCOM::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
    double avg[DIMENSION];
    positions->Average(avg);
    for(int coord=0; coord<DIMENSION; ++coord)
	{
		if(fixedDir[coord])
			avg[coord] = center[coord] - avg[coord];
		else
			avg[coord] = 0.;
	}
    positions->Increment(avg);
}


int PartialFixedCOM::NumberGeneralizedCoordinates()
{
    return DIMENSION*atoms->GetNumber() - nFixed;
}

void PartialFixedCOM::SetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("PartialFixedCOM","SetGeneralizedCoordinates","Not Implemented Yet!");
}


void PartialFixedCOM::GetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("PartialFixedCOM","GetGeneralizedCoordinates","Not Implemented Yet!");
    
}


void PartialFixedCOM::IncrementGeneralizedCoordinate(int n,double inc)
{
    throw DMException("PartialFixedCOM","IncrementGeneralizedCoordinate",
        "Not Implemented Yet!");
}


void PartialFixedCOM::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces,
                                                   double *generalizedForces)
{
    throw DMException("PartialFixedCOM","CalculateGeneralizedForces","Not Implemented Yet!");
}

void PartialFixedCOM::DerivativesWRTGeneralizedCoordinates(int atom,
                                                             double (*gradient)[DIMENSION])
{
    throw DMException("PartialFixedCOM","DerivativesWRTGeneralizedCoordinates",
        "Not Implemented Yet!");
}
