// TimeDependentConstraint.cpp: implementation of the TimeDependentConstraint class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "TimeDependentConstraint.h"
#include "DMArray.h"

#include "DMLastHeader.h"

const std::string TimeDependentConstraint::className = std::string("TimeDependentConstraint");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


TimeDependentConstraint::TimeDependentConstraint(double initialTime, double timeStep, const DMArray<double, DIMENSION> * positions, Functor *func)
{
  positionsZero = NULL;
  Reset(initialTime,timeStep,positions);
  this->func = func;
}


TimeDependentConstraint::~TimeDependentConstraint()
{
  if(positionsZero) delete positionsZero;
}


void TimeDependentConstraint::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
  double functionValue[DIMENSION];
  func->Evaluate(time,functionValue);
  positions->Copy(positionsZero);

  DMArrayIterator<double, DIMENSION> it(positions);
  for(it.Init();!it.IsDone(); it.Next())
    {
      for(int coord=0;coord<DIMENSION;++coord)
		it.Current()[coord] += functionValue[coord];
  }	 
  time += timeStep;
}


void TimeDependentConstraint::Reset(double initialTime, double timeStep, const DMArray<double, DIMENSION> *positions)
{
  time = initialTime;
  this->timeStep = timeStep;

  if(positionsZero) delete positionsZero;
  positionsZero = new DMArray<double, DIMENSION> (positions);
  positionsZero->Copy(positions);
}




Constraint *TimeDependentConstraint::Copy() const
{
  return new TimeDependentConstraint(time, timeStep, positionsZero, func);
}


int TimeDependentConstraint::NumberGeneralizedCoordinates()
{
	return 0;
}

void TimeDependentConstraint::SetGeneralizedCoordinates(double *generalizedCoordinates)
{

}

void TimeDependentConstraint:: GetGeneralizedCoordinates(double *generalizedCoordinates)
{

}

void TimeDependentConstraint::IncrementGeneralizedCoordinate(int n,double inc)
{

}

void TimeDependentConstraint::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{

}

void TimeDependentConstraint::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{

}
