// UniformlyMovingCenterOfMass.cpp: implementation of the UniformlyMovingCenterOfMass class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"

#include "UniformlyMovingCenterOfMass.h"
#include "DMArray.h"
#include "ListOfAtoms.h"

#include "DMLastHeader.h"

const std::string UniformlyMovingCenterOfMass::className = std::string("UniformlyMovingCenterOfMass");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

UniformlyMovingCenterOfMass::UniformlyMovingCenterOfMass(const double velocity[DIMENSION])
{
    this->SetVelocity(velocity);
}

void UniformlyMovingCenterOfMass::SetVelocity(const double velocity[DIMENSION])
{
    for(int coord=0 ; coord<DIMENSION; ++coord) 
        this->velocity[coord] = velocity[coord];
}

void UniformlyMovingCenterOfMass::AdjustVelocities(DMArray<double, DIMENSION> *velocities)
{
    double avg[DIMENSION];
    velocities->Average(avg);
    for(int coord=0; coord<DIMENSION; ++coord)
        avg[coord] = velocity[coord] - avg[coord];
    velocities->Increment(avg);
}

int UniformlyMovingCenterOfMass::NumberGeneralizedCoordinates()
{
    return DIMENSION*(atoms->GetCartesianPositions()->Size() - 1);
}

void UniformlyMovingCenterOfMass::SetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("UniformlyMovingCenterOfMass","SetGeneralizedCoordinates","Not Implemented Yet!");
}


void UniformlyMovingCenterOfMass::GetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("UniformlyMovingCenterOfMass","","Not Implemented Yet!");
}


void UniformlyMovingCenterOfMass::IncrementGeneralizedCoordinate(int n,double inc)
{
    throw DMException("UniformlyMovingCenterOfMass","IncrementGeneralizedCoordinate","Not Implemented Yet!");
}


void UniformlyMovingCenterOfMass::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{
    throw DMException("UniformlyMovingCenterOfMass","CalculateGeneralizedForces","Not Implemented Yet!");
}

void UniformlyMovingCenterOfMass::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{
    throw DMException("UniformlyMovingCenterOfMass","DerivativesWRTGeneralizedCoordinates","Not Implemented Yet!");
}





