#ifndef _DMARRAYITERATOR
#define _DMARRAYITERATOR

#include "DMArray.h"

// ######################################################################
// DMArrayIterator Declaration
// ######################################################################


/*! 
  The iterator is the standard way to traverse a DMArray, because indexing
  with ints is slower. At this time, the iterator has the interface of the 
  Design Pattern. When the iterator is constructed, it copies the
  DMArray's cached list of pointers to the data. As we usual iterator, 
  if the container (DMArray) is changed, the iterator becomes invalidated.
 \ingroup dmsystem
*/

template <class Type, int N=1>
class DMArrayIterator
{
public:
	DMArrayIterator(const DMArray<Type, N> *dmArray);
	~DMArrayIterator() {}
	void Init() ;
	void Next();
	typename ReturnType<Type, N>::IndexOp Current() const {return ReturnType<Type, N>::convert(*current);}
	bool IsDone() const {return done;}
	int Size() const {return size;}
	const Index& GetIndex() const{ return index;}
protected:
	Type (*current)[N];
	Index index;
	unsigned int numberOfLists;
	unsigned int currentLimit;
	int size;
	bool done;
	const std::vector<int> sizes;
	const std::vector<Type (*)[N]> ptrs;
};


// #############################################################################
// ITERATOR Definitions
// #############################################################################



template<class Type, int N>
DMArrayIterator<Type, N>::DMArrayIterator(const DMArray<Type, N> *dmArray) : 
  sizes(dmArray->cache.sizes),ptrs(dmArray->cache.ptrs), size(dmArray->GetShape().GetSize())
{
	numberOfLists = sizes.size();
	done = (size == 0 ? true : false);
}

template<class Type, int N>
void DMArrayIterator<Type, N>::Init()
{
	current = ptrs[0];
	index = 0;
	currentLimit = sizes[0];
	if(currentLimit == 0) Next();
	done = (size == 0 ? true : false);
}


template<class Type, int N>
void DMArrayIterator<Type, N>::Next()
{
	if(++index.inLeaf < currentLimit)
		++current;
	else
	{
		index.inLeaf = 0;
		do
		{
			++index.leaf;
			if(index.leaf == numberOfLists)
			{
				done = true;
				currentLimit = 1; // to make sure to get out of the while loop
			}
			else 
			{
				current = ptrs[index.leaf];
				currentLimit = sizes[index.leaf];
			}
		}
		while(currentLimit == 0);
	}
}



#endif // ! def DMARRAYITERATOR
