#include "DigitalMaterial.h"
#include "DMArray_AnyType.h"
#include "Shape.h"
#include "SynchronizationToken.h"
#include "DMException.h"
#include "DMLastHeader.h"

DMArray_AnyType::DMArray_AnyType( )
{
	//TRACE("DMArray_AnyType::DMArray_AnyType void constructor");
	shape = Shape(0);
	siblings = new SynchronizationToken(this);
}

/*! This constructor may be of the wrong signature.  It should
	be possible to construct a shape only through manipulation
	of smaller objects you create, so you should only be able
	to make leaves of the DMArray.  That is, the argument
	to this constructor should be an integer.  More complicated
	trees are made by addition or cloning.
	*/
DMArray_AnyType::DMArray_AnyType( const Shape& shape )
{
	//TRACE("DMArray_AnyType::DMArray_AnyType Shape constructor");
	this->shape = shape;
	this->siblings = new SynchronizationToken(this);
}

DMArray_AnyType::DMArray_AnyType( const DMArray_AnyType* array )
{
	//TRACE("DMArray_AnyType::DMArray_AnyType sibling constructor");
	bool bSuccess;
	this->shape = array->shape;
	this->siblings = array->siblings;
	bSuccess = this->siblings->AddSibling(this);
	ASSERT( bSuccess );
	if (false == bSuccess) throw DMException("DMArray_AnyType",
		"DMArray_AnyType","Could not add self to list of siblings.");
}

DMArray_AnyType::~DMArray_AnyType()
{
	bool bSuccess;
	if (this->siblings->GetSiblings().size()==1) {
		ASSERT( siblings->Contains(this) );
		delete siblings;
	} else {
		bSuccess = siblings->RemoveSibling(this);
		ASSERT( bSuccess );
		if (false == bSuccess) throw DMException("DMArray_AnyType",
			"~DMArray_AnyType","I wasn't in my own list of siblings.");
	}
}

int DMArray_AnyType::Size() const
{
	return shape.GetSize();
}

const Shape& DMArray_AnyType::GetShape() const
{
	return shape;
}

const DMArray_AnyType::siblingContainer& DMArray_AnyType::GetSiblings() const
{
	return siblings->GetSiblings();
}
