#include "DMFirstHeader.h"
#include <algorithm>
#include "DigitalMaterial.h"
#include "DMArray_AnyType.h"
#include "SynchronizationToken.h"

#include "DMException.h"
#include "DMLastHeader.h"
using namespace std;


/*!	There is no sense making a token unless there is one array.
	*/
SynchronizationToken::SynchronizationToken( DMArray_AnyType*const array )
{
	ASSERT( 0 != array );
	if (0 == array) throw DMException("SynchronizationToken",
		"SynchronizationToken","Passed a null pointer.");

	this->AddSibling( array );
}

/*!	The destructor is empty, but you should always have a destructor
	if you have a constructor.
	*/
SynchronizationToken::~SynchronizationToken( )
{
}

/*!	This does not throw an error if we already have that sibling.
	It does tell you whether the addition succeeded or not, and the only
	reason to fail is that the sibling does not exist.

	\param array The DMArray you want to associate with this set.
	\return Whether it the DMArray could be added.
	*/
bool SynchronizationToken::AddSibling( DMArray_AnyType*const array )
{
	ASSERT( 0 != array );
	if (0 == array) throw DMException("SynchronizationToken","AddSibling",
		"Passed a null pointer.");

	arrayVector::iterator found =
		find(siblings.begin(), siblings.end(), array);
	if(found != siblings.end())
		throw DMException("SynchronizationToken","AddSibling",
			"array already in SynchronizationToken") ;
	else
		siblings.push_back( array );
	
	return true; // return the bool of whether it succeeded.
}

/*! Remove a DMArray from the list of associated arrays.
	\param array Pointer to the array to remove.
	\return Whether we could remove it.  We can't remove it
	if we don't have a pointer to it.
	*/
bool SynchronizationToken::RemoveSibling( DMArray_AnyType*const array )
{
	// If the array is only 1 in size, you shouldn't be removing siblings.
	ASSERT( siblings.size() > 1);
	ASSERT( 0 != array );
	if (0 == array) throw DMException("SynchronizationToken","RemoveSibling",
		"Passed a null pointer.");
	if (1 >= siblings.size()) throw DMException("SynchronizationToken",
		"RemoveSibling","Tried to remove the last pointer to the owner.");

	arrayVector::iterator found =
		find(siblings.begin(), siblings.end(), array);
	if(found == siblings.end())
		throw DMException("SynchronizationToken","AddSibling",
			"array not found in SynchronizationToken") ;
	else
		siblings.erase(found);

	return true; // return the bool of whether it succeeded.
}

/*! Tells you all the siblings of this array.  There will be at least one,
	which is the array itself.  If there are fewer, there is a problem.

	\return An STL container with the siblings.  That should be enough
	information to 
	count the elements and look through it.
	*/
const SynchronizationToken::arrayVector& SynchronizationToken::GetSiblings( ) const
{
	ASSERT(siblings.size()>0);
	return siblings;
}

#ifndef NDEBUG
bool SynchronizationToken::Contains( DMArray_AnyType*const array ) const
{
	ASSERT( 0 != array );
	if (0 == array) throw DMException("SynchronizationToken",
		"Contains","Passed a null pointer.");

	if (siblings.end()== find(siblings.begin(), siblings.end(), array)) return false;
	return true;
}
#endif // NDEBUG
