// MDSimulation.h: interface for the MDSimulation class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MDSIMULATION_H
#define MDSIMULATION_H

#include "DigitalMaterial.h"

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // SWIG

// Base classes here: derived classes in .cpp file
#include "ListOfAtoms.h"
#include "BoundaryConditions.h"
#include "AtomsInitializer.h"
#include "BravaisLatticeWithBasis.h"
#include "AtomsMover.h"
#include "NeighborLocator.h"
#include "Potential.h"
#include "Transformer.h"
#include "SubjectObserver.h"

#include "DM_ComponentList.h"

class MDSimulation  
{
public:
	double GetMaxSize();
	void Reset();
	MDSimulation();
	virtual ~MDSimulation();

	DM_ComponentList listOfListsOfAtoms, listOfAtomsMovers,
		listOfPotentials, listOfBoundaryConditions,
		listOfNeighborLocators, listOfBravaisLattices, listOfAtomsInitializers;
	ComponentSet listOfTransformers, listOfObservers;

#ifndef SWIG
	std::vector<DM_ComponentList *>	listOfLists;

	double const GetDiameter() {return GetPotential()->GetLengthScale();}
	double const GetTimeStep() {return 1.0/sqrt(GetPotential()->GetEnergyCurvatureScale());}	// XXX Depends on the mass
#endif //SWIG

	ListOfAtoms* GetListOfAtoms()
	{return (ListOfAtoms *) listOfListsOfAtoms.GetCurrent();}
	AtomsMover* GetAtomsMover() 
	{return (AtomsMover *) listOfAtomsMovers.GetCurrent();}
	Potential* GetPotential() 
	{return (Potential *) listOfPotentials.GetCurrent();}
	BoundaryConditions* GetBoundaryConditions() 
	{return (BoundaryConditions *) listOfBoundaryConditions.GetCurrent();}
	NeighborLocator* GetNeighborLocator() 
	{return (NeighborLocator *) listOfNeighborLocators.GetCurrent();}
	AtomsInitializer* GetAtomsInitializer() 
	{return (AtomsInitializer *) listOfAtomsInitializers.GetCurrent();}
	BravaisLatticeWithBasis* GetBravaisLattice()
	{return (BravaisLatticeWithBasis *) listOfBravaisLattices.GetCurrent();}
	Transformer* GetTransformer()
	{return (Transformer *) listOfTransformers.GetCurrent();}
	Observer* GetObserver()
	{return (Observer *) listOfObservers.GetCurrent();}

private:
	double center[DIMENSION];
	double rectangleSize[DIMENSION];
	int oldPotentialNumber;	// Ugly! XXX
};

#endif // !defined(MDSIMULATION_H)
