// ExtendedNeighborLocator.cpp: implementation of the ExtendedNeighborLocator class.
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"

#include "ExtendedNeighborLocator.h"
#include "ListOfAtoms.h"
#include "DMException.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ExtendedNeighborLocator::ExtendedNeighborLocator(NeighborLocator *neighborLocator) : NeighborLocator(0.) 
{
	this->neighborLocator = neighborLocator;
}

ExtendedNeighborLocator::~ExtendedNeighborLocator()
{
}

void ExtendedNeighborLocator::SetPositionArray(const DMArray<double, DIMENSION> *positions) 
{
	this->positions = positions;
	DMArray<double, DIMENSION> *pos = const_cast<DMArray<double, DIMENSION>*> (positions);
	pos->Synchronize();
	neighborLocator->SetPositionArray(positions->GetExtended());
}


void ExtendedNeighborLocator::UpdatePosition()
{
	if(!DataIsValid()) UpdateMyData();
}

void ExtendedNeighborLocator::UpdateMyData()
{
	DMArray<double, DIMENSION> *pos = const_cast<DMArray<double, DIMENSION>*> (positions);

	pos->Synchronize();

//	synchronizer->Synchronize(pos);
	neighborLocator->UpdateMyData();
}

bool ExtendedNeighborLocator::DataIsValid()
{
	return neighborLocator->DataIsValid();
}


bool ExtendedNeighborLocator::Neighbors(int atomNumber,
							int * neighbors, 
							int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays)
{
	return neighborLocator->Neighbors(atomNumber, neighbors, numNeighbors, 
							displacements, displacementsSquared, spacesLeftInArrays);
}

bool ExtendedNeighborLocator::HalfNeighbors(int atomNumber,
							int * neighbors, 
							int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays)
{
	return neighborLocator->HalfNeighbors(atomNumber, neighbors, numNeighbors, 
							displacements, displacementsSquared, spacesLeftInArrays);
}

bool ExtendedNeighborLocator::Neighbors(const double *center,
							int* neighbors, 
							int& numNeighbors,
							double (*displacements)[DIMENSION],		// Displacement returns place of other guy wrt main guy
							double* displacementsSquared,
							int& spacesLeftInArrays, double cutoff)
{
	return neighborLocator->Neighbors(center, neighbors, numNeighbors, 
							displacements, displacementsSquared, spacesLeftInArrays, cutoff);
}


bool ExtendedNeighborLocator::Neighbors(int atomNumber, int type1, int type2,
							Index* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays) 
{
	throw DMException("ExtendedNeighborLocator", "Neighbors", "not implemented yet");
	/*
	int nTypes = positions->NumberOfLeaves();
	ASSERT(type1 < nTypes);
	ASSERT(type2 < nTypes);
	bool append = neighborLocator->Neighbors(atomNumber, type1, type2, neighbors, numNeighbors,
		displacements, displacementsSquared, spacesLeftInArrays);
	if(!append) return false;
	int numGhosts;
	int ghostType2 = type2 + nTypes; // if ghosts come after reals
	append = neighborLocator->Neighbors(atomNumber, type1, ghostType2, neighbors+numNeighbors,
		numGhosts, displacements+numNeighbors, displacementsSquared+numNeighbors, spacesLeftInArrays);
	numNeighbors += numGhosts;
	return append;*/
}
