// SimpleNeighborLocator.h: interface for the SimpleNeighborLocator class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SIMPLENEIGHBORLOCATOR_H
#define SIMPLENEIGHBORLOCATOR_H

#include "DigitalMaterial.h"
#include "NeighborLocator.h"

/*!
  SimpleNeighborLocator is the most basic kind of neighbor locator that you could think of:
  Each time it recieves a request to calculate the neighbors of atom (or point) atomNumber,
  it look at all the other points and explicitely computes the distance. It works perfectly
  but it's of course extremely slow. The only case where you could think of using this neighbor 
  locator is if there are a few particles, of if the range of the forces is the same size 
  of the system (or for debugging purposes, if you do not trust the other neighbor locators!).  
*/

class SimpleNeighborLocator : public NeighborLocator 
{
public:
	SimpleNeighborLocator(double cutoffDistance = 1.);
	virtual ~SimpleNeighborLocator();
#ifndef SWIG
	bool HalfNeighbors(int atomNumber, int *neighbors, int &numNeighbors,
							double (*displacements)[DIMENSION], double *displacementsSquared,
							int &spacesLeftInArrays);
	bool Neighbors(int atomNumber, int *neighbors, int &numNeighbors,
							double (*displacements)[DIMENSION], double *displacementsSquared,
							int &spacesLeftInArrays);
	bool Neighbors(const double *center, int* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],	double* displacementsSquared,
							int& spacesLeftInArrays, double cutoff=-1.);
	bool Neighbors(int atomNumber, int type1, int type2,
							int* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays);
	bool HalfNeighbors(int atomNumber, int type1, int type2,
							int* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays);

	NeighborLocator *Copy() const;
	
	void Save(DMWriter *writer) const {NeighborLocator::Save(writer);}
	void Load(DMReader *reader) {NeighborLocator::Load(reader);}
	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}

#endif // SWIG
	void UpdatePosition();

private:
	static const std::string className;
};

#endif // !defined(SIMPLENEIGHBORLOCATOR_H)
