// CheckPointObserver.cpp: implementation of the CheckPointObserver class.
//
//////////////////////////////////////////////////////////////////////


#include "DigitalMaterial.h"
#include "CheckPointObserver.h"
#include "ListOfAtoms.h"
#include "NetCDFWriter.h"
#include "SprintString.h"
#include "BoundaryConditions.h"
#include "NeighborLocator.h"
#include "Potential.h"
#include "AtomsMover.h"

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CheckPointObserver::CheckPointObserver(Potential *potential, AtomsMover *mover, std::string baseFileName, bool oneFile, int notifyLevel) : ListOfAtomsObserver(notifyLevel),lastFileName("")
{
	this->potential = potential;
	this->mover = mover;
	this->baseFileName = baseFileName;
	this->oneFile = oneFile;
	this->counter = 0;
	
	if(oneFile)
		writer = new NetCDFWriter(baseFileName);
	else
		writer = 0;
}

CheckPointObserver::~CheckPointObserver()
{
	if(oneFile)
		delete writer;
}


void CheckPointObserver::Update(ListOfAtoms *atoms)
{

	SprintString suffix(".%.5d",counter);
	// frameName only used oneFile is not true, but need
	// to declare it outside the if statement
	string frameName(baseFileName);
	frameName += suffix;

	
	string LOAName("LOA");
	string BCName("BC");
	string NLName("NL");
	string POTName("POT");
	string MOVName("MOV");

	// if using one file, the frame numbers have to be incorporated into the object names
	if( oneFile )
	{
		LOAName += suffix;
		BCName += suffix;
		NLName += suffix;
		POTName += suffix;
		MOVName += suffix;
	}
	else
	{
		writer = new NetCDFWriter(frameName);
		// cout << frameName << endl;
	}
	


	writer->Put(atoms,LOAName);
	writer->Put(atoms->GetBoundaryConditions(),BCName);
	writer->Put(atoms->GetNeighborLocator(),NLName);
	writer->Put(potential, POTName);
	writer->Put(mover, MOVName);
	
	if( !oneFile )
	{
		delete writer;	
		// it seems wise to only assign lastFileName after successfully
		// doing the save
		lastFileName = frameName;
	}
	++counter;
}

