// CheckPointRestarter.cpp: implementation of the CheckPointRestarter class.
//
//////////////////////////////////////////////////////////////////////


#include "DigitalMaterial.h"
#include "CheckPointRestarter.h"
#include "ListOfAtoms.h"
#include "NetCDFReader.h"
#include "SprintString.h"
#include "BoundaryConditions.h"
#include "NeighborLocator.h"
#include "Potential.h"
#include "AtomsMover.h"

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CheckPointRestarter::CheckPointRestarter(std::string fileName)
{
	reader = new NetCDFReader(fileName);

	potential = 0;
	mover = 0;
	boundaryConditions = 0;
	neighborLocator = 0;
}

CheckPointRestarter::~CheckPointRestarter()
{

}

void CheckPointRestarter::Recreate()
{

	// read the objects off the file
	atoms = dynamic_cast<ListOfAtoms*>(reader->Get("LOA"));
	boundaryConditions= dynamic_cast<BoundaryConditions*>(reader->Get("BC"));
	neighborLocator = dynamic_cast<NeighborLocator*>(reader->Get("NL"));
	potential = dynamic_cast<Potential*>(reader->Get("POT"));
	mover = dynamic_cast<AtomsMover *>(reader->Get("MOV"));
	
	// now reconnect everything
	atoms->SetBoundaryConditions(boundaryConditions);
	atoms->SetNeighborLocator(neighborLocator);
	mover->SetPotential(potential);

	// This doesn't work for all movers! In particular it won't work for a mover which has a pointer to another object. CompositeAtomsMover is one such, but it has not been developed very far anyway. CompositeLangevinAtomsMover will work when I have added in code to save the friction array.

}
