// GravityPotential.h: interface for the GravityPotential class.
//
//////////////////////////////////////////////////////////////////////

#ifndef GRAVITYPOTENTIAL_H
#define GRAVITYPOTENTIAL_H

#include "DMFirstHeader.h"

#include "DigitalMaterial.h"
#include "Potential.h"

class GravityPotential : public Potential
{
public:
	GravityPotential();
	virtual ~GravityPotential() {};

	void CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces);
	double CalculateEnergy(ListOfAtoms *atoms);	

	double GetG() const {return g;}
	void SetG(double g);

 	void SetDirection(double direction[DIMENSION]);


	double GetLengthScale() const {return 1.;} // this is arbitrary !!!
	double GetEnergyCurvatureScale() const {return 1.;} // Arbitrary! Should be zero.
	double CalculateAtomicEnergy(ListOfAtoms *atoms, int i) {return 0;}
	
	void Save(DMWriter *writer) const
		{    
			double doubleDir[DIMENSION];
			std::vector<int> dim;
			dim.push_back(DIMENSION);
			writer->PutDoubleArray(doubleDir, dim, "direction");
			writer->PutDouble(g,"g");
		}
	void Load(DMReader *reader)
		{
			std::vector<int> dim;
			double *data = reader->GetDoubleArray(dim, "direction");
			if((dim.size() != 1) || (dim[0] != DIMENSION)) 
				throw DMException("GravityPotential", "Load", "Problem while reading direction");
			for(int coord=0; coord<DIMENSION; ++coord) direction[coord] = data[coord];
			delete data;
			g = reader->GetDouble("g");
		}

	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}

#ifndef SWIG
private:
	double g;
	double direction[DIMENSION];

	static const std::string className;
#endif // SWIG 
};

#endif // !defined(GRAVITYPOTENTIAL_H
