// AnisotropicDislocationMaker.h: interface for the AnisotropicDislocationMaker class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ANISOTROPICDISLOCATIONMAKER_H
#define ANISOTROPICDISLOCATIONMAKER_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <complex>


#include "DislocationMaker.h"
#include "ListOfAtoms.h"

//! Introduces straight dislocations into materials with cubic anisotropy

/*!

  AnisotropicDislocationMaker inherits from DislocationMaker, overloading the
  ElasticDisplacement and GradientWRTPosition functions. At the moment (Feb 
  2001), it only deals with cubic anisotropy, and only when the dislocation 
  line is along a cubic axis of the material.

  \date January 2001
  \author Nicholas Bailey

 */

class AnisotropicDislocationMaker : public DislocationMaker
{
public:
	AnisotropicDislocationMaker(const double burgers[DIMENSION],const double direction[DIMENSION], double center[DIMENSION],double *elasticConstats,double thetaLowerCutoff = -PI);
	virtual ~AnisotropicDislocationMaker();

	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);


	void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);

	
#ifndef SWIG

private:
//	void UpdateMemberVariables();
	double elasticConstants[3]; // assuming cubic symmetry: 3 elastic constants (C11, C12, C44)	
	std::complex<double> anisotropyPhaseFactor; // e^(i alpha) in Eshelby, Sinclair papers
	std::complex<double> logCoefficient; // D/(2 Pi i) in the papers
	std::complex<double> yCompFactor; // A in Eshelby, Sinclair papers
#endif //SWIG
};

#endif // !defined(ANISOTROPICDISLOCATIONMAKER_H)
