// AnisotropicMultipoleField.h: interface for the AnisotropicMultipoleField class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ANISOTROPICMULTIPOLEFIELD_H
#define ANISOTROPICMULTIPOLEFIELD_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <complex>

#include "MultipoleField.h"


class AnisotropicMultipoleField : public MultipoleField
{
public:
	AnisotropicMultipoleField(int maximumOrder, const double direction[DIMENSION], double center[DIMENSION],double *C);
	virtual ~AnisotropicMultipoleField();

	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);
	void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);

#ifndef SWIG
	void GradientWRTParameters(const double position[DIMENSION], double (*displacementGradient)[DIMENSION]);

private:
	double elasticConstants[3]; // assuming cubic symmetry: 3 elastic constants (C11, C12, C44)	
	std::complex<double> anisotropyPhaseFactor; // alpha in Eshelby, Sinclair papers
	std::complex<double> yCompFactor; // A in Eshelby, Sinclair papers
#endif //SWIG
};

#endif // !defined(ANISOTROPICMULTIPOLEFIELD_H)
