// CopySubset.cpp: implementation of the CopySubset class.
//
//////////////////////////////////////////////////////////////////////

#include "DigitalMaterial.h"
#include "CopySubset.h"
#include "AtomsLocator.h"
#include <vector>
#include <algorithm>

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CopySubset::CopySubset( AtomsLocator *locator)
{
	this->locator = locator;
}

CopySubset::~CopySubset()
{

}


/*! The Transform function makes a copy of the positions, iterates over the 
  atom numbers in listOfAtomNumbers, gets the position of that atom in the
  copied array, calculates the displacement, and increments
  the position (in the copy). Then we use SetCartesianPositions to change
  the positions in the LOA. I'm assuming this is faster than using
  IncrementCartesianPosition for each atom to be moved--the neighborLocator is 
  called just once, but all atoms outside the subset get copied over 
  identically

  
  It would be great if we could set up a mechanism for
  IncrementCartesianPositions to work on a subset of the LOA, perhaps by 
  creating an LOA which is essentially a reference into the given one.


 */

ListOfAtoms *CopySubset::Copy(ListOfAtoms *atoms)
{
	std::vector<int> atomsToKeep = locator->GetAtomNumbers(atoms);
	sort(atomsToKeep.begin(),atomsToKeep.end());
	
	int nAtomsTotal = atoms->GetNumber();
	int nAtomsInSubset = atomsToKeep.size();


	if(nAtomsInSubset == 0)
		return new ListOfAtoms(0);
	

	ListOfAtoms *copy = atoms->Copy();
	
	std::set<int> atomsToRemove;
	int i = 0;
	std::vector<int>::const_iterator toKeepIter = atomsToKeep.begin();
	
	
	int nextAtomToKeep;
	
	while(toKeepIter != atomsToKeep.end())
	{

	    nextAtomToKeep = *toKeepIter;
		for ( int at = i; at < nextAtomToKeep; ++at )
			atomsToRemove.insert(at);


		i = nextAtomToKeep + 1;
		toKeepIter++;
	};
	// now include all the atoms after the last "to keep" one.
	for ( int at = nextAtomToKeep + 1; at < nAtomsTotal; ++at )
		atomsToRemove.insert(at);

	copy->RemoveAtoms(atomsToRemove);

	return copy;
}

