// ElasticFieldTransformer.cpp: implementation of the ElasticFieldTransformer
// class.
//
//////////////////////////////////////////////////////////////////////

#include "DMFirstHeader.h"
#include <fstream>
#include "DigitalMaterial.h"

#include "ElasticFieldTransformer.h"
#include "DMArray.h"

#include "DMLastHeader.h"

void ElasticFieldTransformer::Transform(ListOfAtoms *atoms)
{
    if(atoms->GetNumber() == 0) return;
    int coord;

	double *position, *posIncrement;
    double displacement[DIMENSION];
	
	DMArrayIterator<double,DIMENSION> posIter(atoms->GetCartesianPositions());
	DMArray<double, DIMENSION> increments(atoms->GetShape());
	DMArrayIterator<double,DIMENSION> incIter(&increments);

	for(posIter.Init(),incIter.Init(); !posIter.IsDone(); posIter.Next(),incIter.Next())	
	{
		position = posIter.Current();
		posIncrement = incIter.Current();
		
        for(coord=0;coord<DIMENSION;++coord)
            displacement[coord] = 0.;
		
        ElasticDisplacement(position, displacement);
		
		for(coord=0;coord<DIMENSION;++coord)
			posIncrement[coord] = displacement[coord];
	}

	atoms->IncrementCartesianPositions(&increments);    
}

