%{
#include "AnisotropicMultipoleField.h"
#include "DislocationMaker.h"
#include "EulerCoordinateTransformer.h"
#include "IsotropicMultipoleField.h"
#include "LinearPositionTransformer.h"
#include "NotchFieldDisplacer.h"
#include "SubsetElasticTransformer.h"
%}



/* this typemap allows us to call ElasticDisplacement and access the new value of the displacement as a Python in the return value. Another typemap handles the passing of double displacement[DIMENSION] as an input argument*/
%typemap(python,argout) double displacement[DIMENSION]   
{
	PyObject *dispList;
	dispList = PyList_New(0);
	for(int coord=0;coord<DIMENSION;++coord)
	PyList_Append(dispList,PyFloat_FromDouble($1[coord]));
	if((!$result) || ($result == Py_None))
		$result = dispList;
	else
	{
		if(!PyList_Check($result))
		{
			PyObject *oldReturnValue = $result;
			$result = PyList_New(0);
			PyList_Append($result,oldReturnValue);
			Py_XDECREF(oldReturnValue);
		}
		PyList_Append($result,dispList);
		Py_XDECREF(dispList);
	}

}

%nodefault ElasticFieldTransformer;
class ElasticFieldTransformer: public Transformer {};
%nodefault Quasi2DElasticField;
class Quasi2DElasticField : public ElasticFieldTransformer 
{
public:
    void SetCutoff(double cutoff);
    double GetCutoff();	
};


%nodefault MultipoleField;
class MultipoleField : public Quasi2DElasticField
{
public:
int GetMaximumOrder() const;

%extend
{
	void SetCoefficients(PyObject *pyObj)
	{
		int maximumOrder = self->GetMaximumOrder();
		if(PythonListDouble2DCheck(pyObj, maximumOrder, 4))
		{
		double (*coefficients)[4] = new double[maximumOrder][4];
		for( int a = 0; a < maximumOrder; ++a )
		{
			PyObject *row = PyList_GetItem(pyObj,a);
			for( int b = 0; b < 4; ++b )
			{
				PyObject *element = PyList_GetItem(row,b);
				coefficients[a][b] = PyFloat_AsDouble(element);
			}
		}
		self->SetCoefficients(coefficients);
		delete [] coefficients;	
		}
 		else if( PythonArrayDouble2DCheck(pyObj, maximumOrder, 4))
		{
		double (*coefficients)[4] = new double[maximumOrder][4];
		PyArrayObject *array = (PyArrayObject *) pyObj;
		for(int a = 0; a < maximumOrder; ++a )
		for( int b = 0; b < 4; ++b )
			coefficients[a][b] = *(double*)(array->data + a * array->strides[0] + b * array->strides[1]);

		self->SetCoefficients(coefficients);
		delete [] coefficients;	
		}
		else PyErr_SetString(PyExc_TypeError,"passed object is neither array nor list");
		
	}
}
};


%include "DislocationMaker.h"
%include "IsotropicMultipoleField.h"
%include "EulerCoordinateTransformer.h"
%include "LinearPositionTransformer.h"
%include "IsotropicMultipoleField.h"
%include "AnisotropicMultipoleField.h"
%include "SubsetElasticTransformer.h"

class NotchFieldDisplacer : public Quasi2DElasticField
{
public:
	NotchFieldDisplacer(const double direction[DIMENSION], double center[DIMENSION], ArrayDoubleVector *strohVectorsReal, ArrayDoubleVector *strohVectorsImag, double strohEigenValuesReal[DIMENSION], double strohEigenValuesImag[DIMENSION],double notchAlphaAngle, double lambdaExponent[2], double stressIntensity[2]);

	// Gets and Sets for member variables
	void SetStressIntensity( double stressIntensity[2] );
	double GetStressIntensity(int a) { return stressIntensity[a]; }
};
