// IncrementalMinimizer.h: interface for the IncrementalMinimizer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef INCREMENTALMINIMIZER_H
#define INCREMENTALMINIMIZER_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include "Transformer.h"
class ElasticFieldTransformer;

template<class Type, int N> class DMArray;
class Potential;

//! This transformer will adjust a minimized LOA so to be the first order approximation to the minimum under an additional perturbing external force


/**
 * This transformer will take an LOA that has been minimized, and generate
 * corrections to the positions that minimize it under the action of an 
 * additional small external force (possibly something more general later)
 *
 * \ingroup moleculardynamics
 * \author nbailey
 * \date May 17, 2001
 */


class IncrementalMinimizer : public Transformer
{
public:
	IncrementalMinimizer( int mode, Potential *potential,  double epsilon );
	virtual ~IncrementalMinimizer();

	void SetMode(int mode){ this->mode = mode; }
	int GetMode() const { return mode; }
	
	void SetEpsilon(double epsilon) { this->epsilon = epsilon; }
	double GetEpsilon() const { return epsilon; }
	
	void SetForceIncrements(const DMArray<double, DIMENSION> *forceIncrements);
	const DMArray<double, DIMENSION> *GetForceIncrements() const { return forceIncrements; }

	void SetPositionIncrements(int constrainedBranch, const DMArray<double, DIMENSION> *positionIncrements);
const DMArray<double, DIMENSION> *GetPositionIncrements() const { return positionIncrements; }

	void Verbose() { verbose = true; }
	void Quiet() { verbose = false; }

	virtual void Transform(ListOfAtoms *atoms);
 private:
	void Allocate(int nAtoms);
	void SolveLinearSystem(double *arrayForLinearSolver, double *positionCorrections, int matrixSize);
	void CalculateNumericalHessian(ListOfAtoms *atoms);
	int mode;
	const DMArray <double, DIMENSION> *forceIncrements;
	const DMArray <double, DIMENSION> *positionIncrements;
	int constrainedBranch;	

	double *forceDerivatives;
	double *positionCorrections;
	int nAtoms;
	Potential *potential;
	double epsilon;
	bool verbose;
};


#endif // !defined(INCREMENTALMINIMIZER_H)
