// MultipoleField.h: interface for the MultipoleField class.
//
//////////////////////////////////////////////////////////////////////


#ifndef MULTIPOLEFIELD_H
#define MULTIPOLEFIELD_H


#include "Quasi2DElasticField.h"

class MultipoleField : public Quasi2DElasticField
{
public:
	MultipoleField(int maximumOrder, const double direction[DIMENSION], double center[DIMENSION]):Quasi2DElasticField(direction,center)
		{
			this->maximumOrder = maximumOrder;
			coefficients = new double[maximumOrder][4];
			for(int n=0;n<maximumOrder;++n)
				for(int j=0;j<4;++j)
					coefficients[n][j] = 0.;
		}
	virtual ~MultipoleField()
		{
			delete [] coefficients;
		}
	virtual void GradientWRTParameters(const double position[DIMENSION], double (*displacementGradient)[DIMENSION])=0;

	void SetCoefficients(double (*coefficients)[4])
		{
			for(int n=0;n<maximumOrder;++n)
				for(int j=0;j<4;++j)
					this->coefficients[n][j] = coefficients[n][j];
		}
	
	void GetCoefficients(double (*coefficients)[4])
		{
			for(int n=0;n<maximumOrder;++n)
				for(int j=0;j<4;++j)
					coefficients[n][j] = this->coefficients[n][j];
		}
	int GetMaximumOrder() const {return maximumOrder;}

protected:
	int maximumOrder;
	double (*coefficients)[4];

};


#endif // !defined(MULTIPOLEFIELD_H)
