// Quasi2DElasticField.h: interface for the Quasi2DElasticField class.
//
//////////////////////////////////////////////////////////////////////

#ifndef QUASI2DELASTICFIELD_H
#define QUASI2DELASTICFIELD_H


#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <assert.h>

#include "ElasticFieldTransformer.h"
#include "ListOfAtoms.h"


class Quasi2DElasticField : public ElasticFieldTransformer
{
public:
	Quasi2DElasticField(const double direction[DIMENSION], double center[DIMENSION],double thetaLowerCutoff = 0);

	virtual void CalculateGeometry(const double position[DIMENSION]);

	virtual void SetDirection(const double direction[DIMENSION]); 
	virtual const double *GetDirection() const {return (const double *)direction;}
	virtual void SetCenter(const double center[DIMENSION]); 
	virtual const double *GetCenter() const {return (const double *)center;}
	void SetCutoff(const double cutoff){this->cutoff = cutoff;}
	double GetCutoff() const {return cutoff;}

	void SetOrientingVector(int a,const double vector[DIMENSION]){for(int coord=0;coord<DIMENSION;++coord)orientingVector[a][coord] = vector[coord];}
	const double *GetOrientingVector(int a){return orientingVector[a];}

	void GradientWRTCenter(const double position[DIMENSION],double gradientWRTCenter[DIMENSION][DIMENSION]);

protected:
	double direction[DIMENSION];
	double center[DIMENSION];
	double orientingVector[2][DIMENSION];
	double cutoff; // cutoff the singularity---don't displace atoms closer than this to the line
	double parallelComp[DIMENSION];
	double perpComp[DIMENSION];
	double rPerp;
	double rPerpInverse;
	double theta;
	double thetaLowerCutoff; // determines which 2 PI range of theta is allowed
	double cosTheta;
	double sinTheta;
	double orient1Comp;
	double orient2Comp;	

};

#endif // !defined(QUASI2DELASTICFIELD_H)
