// RectangleAtomsLocator.h: interface for the RectangleAtomsLocator class.
//
//////////////////////////////////////////////////////////////////////

#ifndef RECTANGLEATOMSLOCATOR_H
#define RECTANGLEATOMSLOCATOR_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "AtomsLocator.h"


//! Identifies atoms in a rectangular regoin



/*!
  Constructor arguments include the bounding faces and a boolean 'bounded' for 
  each direction. If this is false then the bounds in that direction are
  ignored.
  

  \author nbailey
  \date 2/25/2002
  \ingroup moleculardynamics

 */


class RectangleAtomsLocator : public AtomsLocator
{
public:
	RectangleAtomsLocator(double lower[DIMENSION], double upper[DIMENSION], bool bounded[DIMENSION] );
	virtual ~RectangleAtomsLocator();
	bool Inside(const double *position);


	void SetLower(double lower[DIMENSION]) 
		{
			for(int coord = 0; coord < DIMENSION; ++coord ) 
				this->lower[coord] = lower[coord];
			CalculateCenterRadius();
		}
	void SetUpper(double upper[DIMENSION])
		{
			for(int coord = 0; coord < DIMENSION; ++coord )
				this->upper[coord] = upper[coord];CalculateCenterRadius();
			CalculateCenterRadius();
		}
	
	void SetBounded(bool bounded[DIMENSION])
		{
			for(int coord = 0; coord < DIMENSION; ++coord )
				this->bounded[coord] = bounded[coord];	
		}

#ifndef SWIG
	const std::vector<int> &GetAtomNumbers(ListOfAtoms *atoms);
	
private:
	void AllocateForNeighborLocator(int nSpaces);
	void CalculateCenterRadius();
	double lower[DIMENSION];
	double upper[DIMENSION];
	bool bounded[DIMENSION]; 
	double center[DIMENSION];
	double radius;
	
	int nSpaces;
	double *displacementsSq;
	double (*displacements)[DIMENSION];
	int *neighbors;
#endif // SWIG
};

#endif // !defined(RECTANGLEATOMSLOCATOR_H)
