// SliceAtomsLocator.h: interface for the SliceAtomsLocator class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SLICEATOMSLOCATOR_H
#define SLICEATOMSLOCATOR_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "AtomsLocator.h"
#include <cmath>

//! Identifies atoms in "slice" region---specified by a center, a normal and a thickness



/*!
  Constructor arguments include a center, which is any point in the mid-plane 
  of the slice, a unit vector which is the normal to the mid-plane, and the 
  slice thickness.
  

  \author nbailey
  \date 7/12/2002
  \ingroup moleculardynamics

 */


class SliceAtomsLocator : public AtomsLocator
{
public:
	SliceAtomsLocator(double center[DIMENSION], double normal[DIMENSION], double thickness);
	virtual ~SliceAtomsLocator();
	bool Inside(const double *position);


	void SetCenter(double center[DIMENSION]) 
		{
			for(int coord = 0; coord < DIMENSION; ++coord ) 
				this->center[coord] = center[coord];
			//CalculateCenterRadius();
		}
	void SetNormal(double normal[DIMENSION])
		{
			int coord;
			double norm = 0.;
			for(coord = 0; coord < DIMENSION; ++coord )
			{
				this->normal[coord] = normal[coord];
				norm += this->normal[coord] * this->normal[coord];
			}
			norm = sqrt(norm);

			for(coord = 0; coord < DIMENSION; ++coord )
				this->normal[coord] /= norm;
		}
	
	void SetThickness(double thickness)
		{
			this->thickness = fabs(thickness);	
		}

#ifndef SWIG
private:
	double center[DIMENSION];
	double normal[DIMENSION];
	double thickness;

#endif // SWIG
};

#endif // !defined(SLICEATOMSLOCATOR_H)
