// SubsetElasticTransformer.h: interface for the SubsetElasticTransformer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SUBSETELASTICTRANSFORMER_H
#define SUBSETELASTICTRANSFORMER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "ElasticFieldTransformer.h"
class AtomsLocator;


//! This will apply an ElasticFieldTransformer to a subset (specified by atom numbers) of the LOA


/*!

  You pass to the constructor pointers to two objects: one is an
  ElasticFieldTransformer which specifies which kind of displacement is to be
  applied to the selected atoms. The other is an AtomsLocator, which has a
  function GetAtomNumbers which provides a vector of the atom numbers to which
  the transformation is to be applied.

  \author nbailey
  \date 7/10/2001
  \ingroup moleculardynamics

 */


class SubsetElasticTransformer : public ElasticFieldTransformer
{
public:
	SubsetElasticTransformer(ElasticFieldTransformer *transformerToApply, AtomsLocator *locator);
	virtual ~SubsetElasticTransformer();

	void Transform(ListOfAtoms *atoms);

	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);

    void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);
	
private:
#ifndef SWIG
	ElasticFieldTransformer *transformerToApply;
	AtomsLocator *locator;
#endif // SWIG
};

#endif // !defined(SUBSETELASTICTRANSFORMER_H)
