// UndercoordinatedPruner.h: interface for the UndercoordinatedPruner class.
//
//////////////////////////////////////////////////////////////////////

#ifndef UNDERCOORDINATEDPRUNER_H
#define UNDERCOORDINATEDPRUNER_H

#include "DMFirstHeader.h"
#include <vector>

#include "Transformer.h"

/*
  The duty of this class is to remove atoms which are undercoordinated, e.g. 
  Silicon atoms with only one neighbor.

  \author Nicholas Bailey
  \date August 2001
  \ingroup moleculardynamics
*/

class ListOfAtoms;

class UndercoordinatedPruner : public Transformer  
{
public:
	UndercoordinatedPruner(double cutoff, int minCoordination = 2);
	virtual ~UndercoordinatedPruner();
	void SetCutoff(double cutoff);
	double GetCutoff() {return cutoff;}
	void Transform(ListOfAtoms *atoms);
#ifndef SWIG
	const std::set<int> &GetAtomsRemoved() {return atomsRemoved;}
private:

	double cutoff;
	int minCoordination;
	std::set<int> atomsRemoved;
#endif //SWIG
};

#endif // !defined(UNDERCOORDINATEDPRUNER_H)
