// NotchMaker.cpp: implementation of the WedgeMaker class.
//
//////////////////////////////////////////////////////////////////////
#include "DigitalMaterial.h"
#include "WedgeMaker.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WedgeMaker::WedgeMaker(double direction[DIMENSION], double center[DIMENSION], double normal1[DIMENSION], double normal2[DIMENSION])
{

	for(int coord=0; coord<DIMENSION; coord++){
		this->direction[coord] = direction[coord];
		this->center[coord] = center[coord];
		this->normal1[coord] = normal1[coord];
		this->normal2[coord] = normal2[coord];
	}
}

WedgeMaker::~WedgeMaker()
{

}

void WedgeMaker::SetDirection(const double direction[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->direction[coord] = direction[coord];
}

void WedgeMaker::SetCenter(const double center[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->center[coord] = center[coord];
}

void WedgeMaker::SetNormal1(const double normal1[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->normal1[coord] = normal1[coord];
}

void WedgeMaker::SetNormal2(const double normal2[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->normal2[coord] = normal2[coord];
}

bool WedgeMaker::Inside(const double X[DIMENSION])
{
	bool inside;
	double perpDist[DIMENSION];
	double XdotDir = 0;
	double perpdotNorm1 = 0;
	double perpdotNorm2 = 0;

	for (int coord = 0; coord < DIMENSION; coord++)
		XdotDir += (X[coord]-center[coord])*direction[coord];
	for (int coord = 0; coord < DIMENSION; coord++){
		perpDist[coord] = X[coord] - center[coord] - XdotDir*direction[coord];
		perpdotNorm1 += perpDist[coord]*normal1[coord];
		perpdotNorm2 += perpDist[coord]*normal2[coord];
	}
	inside = ((perpdotNorm1 >= 0.0) && (perpdotNorm2 >= 0.0));

	return inside;
}

