// RasMolObserver.h: interface for the RasMolObserverCPP class.
//
//////////////////////////////////////////////////////////////////////

#ifndef RASMOLOBSERVER_H
#define RASMOLOBSERVER_H

#include <stdio.h> // for FILE
#include <map>
#include "ListOfAtomsObserver.h"
#include "ColorMethod.h"

//! Uses RasMol to plot the atoms
/*! It is  an observer of ListOfAtoms. Copied from PlotAtomsObserver

  Possibly would help if could get automatic temporary local files for the data, as is done in Python.

  \date Feb 1, 2001
  \author Nicholas Bailey

  */

class RasMolObserverCPP : public ListOfAtomsObserver  
{

public:

  //! Constructor.
	RasMolObserverCPP(int radius,ColorMethod *colorMethod=NULL);
  //! Destructor.
	virtual ~RasMolObserverCPP();

  //! Called by the subject to tell us when atoms move.
	void Update(ListOfAtoms *atoms);

  //! Alternative syntax for Update().
	void Plot(ListOfAtoms *atoms);

	void SetColor(int leaf,int color)
		{
			colorMap.insert(std::make_pair(leaf,color));
		}
	
	void SetColorScale ( double colorScale ) { this->colorScale = colorScale; colorScaleSetByUser = true; }
	void UnSetColorScale() { colorScaleSetByUser = false; }
	double GetColorScale() { return colorScale; }
	
	
	void SetLengthScale ( double lengthScale ) { this->lengthScale = lengthScale; }
	double GetLengthScale() { return lengthScale; }
protected:
#ifndef SWIG
	FILE*	        pipe_file; //!< Pipe to RasMol.
	int             radius;
	ColorMethod*    colorMethod;
	double colorScale;
	bool colorScaleSetByUser;
	std::map<int, int> colorMap; // maps leaf index to colour index

	double lengthScale;
#endif // SWIG	

};

#endif // !defined(RASMOLOBSERVER_H)
