// ChangeParameterMenu.h: interface for the ChangeParameterMenu class.
//
// Drew Dolgert 15 Feb 2000
// The ComponentSelector is responsible for associating commands (ID numbers
// for Windows commands here) with which objects in the simulation
// are selected to run.
//
// This is separate from any menus, which associate a graphical selection
// with an ID through an ID to MenuHandle map.
//
// There is one ComponentSelector for each simulation.  There can be many
// menus which refer to the ComponentSelector.
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ChangeParameterMenu_H__68D440FA_C5B2_4F33_99C5_B41A1FE10414__INCLUDED_)
#define AFX_ChangeParameterMenu_H__68D440FA_C5B2_4F33_99C5_B41A1FE10414__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include <map>
#include "Introspective.h"
#include "MDSimulation.h"

class Thread;

/*! \ingroup windows */
class ChangeParameterMenu  
{
public:
	ChangeParameterMenu(MDSimulation *sim, Thread *simThread);
	virtual ~ChangeParameterMenu();
	virtual HMENU GetMenu(); // get the menu to append to title menu
	virtual void OnSelect(UINT idd); // responds to menu command
	virtual void OnUpdateComponentUI (CCmdUI* pCmdUI); // Called to decide whether to put a check mark.
private:
	virtual bool IsSelected(UINT idd) const; // How we query the sim to ask whether to put a check.
	virtual HMENU MakeComponentList(MDSimulation *sim);

	struct ComponentIdentifier {
		int iBaseType;
		int iType;
	};
	MDSimulation *pSim;
	Thread * pSimThread;
	HMENU selectMenu; // the menu folks ask for.

	// Assign own identification number to dialog box class
	typedef std::map<UINT,ComponentIdentifier> CommandToComponentMap;
	CommandToComponentMap menuMap;
};

#endif // !defined(AFX_ChangeParameterMenu_H__68D440FA_C5B2_4F33_99C5_B41A1FE10414__INCLUDED_)
