// SimThreadController.cpp: implementation of the SimThreadController class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MD.h"
#include "SimThreadController.h"
#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SimThreadController::SimThreadController()
{
	pauseThread = false;
	stopThread = true;	// Must announce to SimThreadController just before starting thread: call Started()
}

SimThreadController::~SimThreadController()
{

}

void SimThreadController::Pause()
{
	if (ThreadRunning()) // Don't try to pause already halted threads
	{
		pauseThread = true;
		threadPaused.Lock();	// Wait for thread to pause
		threadPaused.Unlock();	// Probably unnecessary: conforms to Microsoft tradition
	}
}

void SimThreadController::Resume()
{
	assert(pauseThread=true);	// Should know thread is paused
	threadResume.SetEvent();	// Signal thread to start again
}

void SimThreadController::Paused()
{
	threadPaused.SetEvent();	// Thread announces that it's paused
	threadResume.Lock();		// Wait for main program to allow restart
	threadResume.Unlock();		// Probably unnecessary: conforms to Microsoft tradition
	pauseThread = false;
}

bool SimThreadController::PauseNow() const
{
	return pauseThread;
}

void SimThreadController::Stop()
{
	if (!stopThread)	// Check if thread is still running
	{
		stopThread=true;
		if (pauseThread)	// Check if thread is paused: need to restart to successfully end
			Resume();
		threadStopped.Lock();	// Wait for thread to stop
		threadStopped.Unlock();	// Probably unnecessary: conforms to Microsoft tradition
	}
}

void SimThreadController::Stopped()
{
	threadStopped.SetEvent();	// Thread announces it's demise
}

bool SimThreadController::StopNow() const
{
	return stopThread;
}

void SimThreadController::Started()
{
	stopThread = false;
}

bool SimThreadController::ThreadRunning()
{
	return (!stopThread && !pauseThread);
}