#include "vtkSphereFilter.h"
#include "MDSimulation.h"
#include "ListOfAtoms.h"

vtkSphereFilter::vtkSphereFilter(MDSimulation *sim)
{
	this->Mapper = vtkPolyDataMapper::New();
	this->Actor = vtkActor::New();
//	this->Actor->SetMapper(this->Mapper);
//	this->Actor->VisibilityOff();
	this->vtkPropSource::GetProps()->AddItem(this->Actor);
	pAtomPolyData = vtkPolyData::New();
	pAtomCoords = vtkFloatPoints::New();

	ball = vtkSphereSource::New();
	ball->SetRadius(0.35);
	ball->SetThetaResolution(12);
	ball->SetPhiResolution(12);
	
	balls = vtkGlyph3D::New();
//	balls->SetSource(ball->GetOutput());
//	balls->SetInput(pAtomPolyData);

//	this->Mapper->SetInput(balls->GetOutput());
//	this->Actor->VisibilityOn();
	ChainIntegrity();
	pAtoms = sim->GetListOfAtoms();
	pAtoms->Attach(this);
}

void vtkSphereFilter::ChainIntegrity()
{
	this->Actor->VisibilityOff();
	pAtomPolyData->SetPoints(pAtomCoords);

	balls->SetSource(ball->GetOutput());
	balls->SetInput(pAtomPolyData);

	this->Mapper->SetInput(balls->GetOutput());
	this->Actor->SetMapper(this->Mapper);
	this->Actor->VisibilityOn();
}

vtkSphereFilter::~vtkSphereFilter()
{
	if (Mapper) this->Mapper->Delete();
	if (Actor) this->Actor->Delete();
	if (pAtomPolyData) pAtomPolyData->Delete();
	if (pAtomCoords) pAtomCoords->Delete();
	if (ball) ball->Delete();
	if (balls) balls->Delete();

}

void vtkSphereFilter::Update(Subject *theSubject)
{
	static int i = 0;
	ListOfAtoms *atoms = dynamic_cast<ListOfAtoms*>(theSubject);
	i++;
	this->pAtoms = atoms;
	this->bDirty = true;
	if (i<5) this->Notify();
	this->GetCurrentPolyData();
}

vtkPropCollection *vtkSphereFilter::GetProps()
{
	if (bDirty && pAtoms) GetCurrentPolyData();
	return this->vtkPropSource::GetProps();
};


void vtkSphereFilter::GetCurrentPolyData()
{
	int nAtoms = pAtoms->GetNumber();
	if (pAtomCoords->GetNumberOfPoints()!=nAtoms) {
		pAtomCoords->Reset();
		pAtomCoords->SetNumberOfPoints(nAtoms);
	}
	DMArrayIterator dpAtoms(pAtoms->GetCartesianPositions());

	int i;
	float dTemp[3] = {0,0,0};
	pAtoms->ReadLock();
	dpAtoms.Init();
	for (i=0; i<nAtoms; i++) {
		for (int j=0; j<DIMENSION; j++) dTemp[j]=(*dpAtoms)[j];
		pAtomCoords->SetPoint(i,dTemp);
		dpAtoms.Next();
	}
	pAtoms->ReadRelease();
	this->bDirty = false;
	// This next line copies the coordinates into the polydata.
	// If we can make it thread-safe, then the whole thing is.
//	vtkPolyData *pTempAtomPolyData = vtkPolyData::New();
//	pTempAtomPolyData->SetPoints(pAtomCoords);
//	balls->SetInput(pTempAtomPolyData);
//	if (pAtomPolyData) pAtomPolyData->Delete();
//	pAtomPolyData = pTempAtomPolyData;
	pAtomPolyData->SetPoints(pAtomCoords);
	balls->SetInput(pAtomPolyData);
}

